@@ -29,8 +29,11 @@ import java.util.HashMap;
 import java.util.Locale;
 import java.util.Map;
 import java.util.Properties;
+import java.util.concurrent.ThreadFactory;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicLong;
+
+import com.google.common.util.concurrent.ThreadFactoryBuilder;
 import org.apache.commons.lang.StringUtils;
 import org.apache.http.HttpHost;
 import org.apache.http.auth.AuthSchemeProvider;
@@ -148,7 +151,7 @@ public class ElasticSearchAuditDestination extends AuditDestination {
                         addFailedCount(1);
                         logFailedEvent(Arrays.asList(itemRequest), itemResponse.getFailureMessage());
                     } else {
-                        if(LOG.isDebugEnabled()) {
+                        if (LOG.isDebugEnabled()) {
                             LOG.debug(String.format("Indexed %s", itemRequest.getEventKey()));
                         }
                         addSuccessCount(1);
@@ -188,7 +191,7 @@ public class ElasticSearchAuditDestination extends AuditDestination {
         if (subject != null) {
             KerberosTicket ticket = CredentialsProviderUtil.getTGT(subject);
             try {
-                if (new Date().getTime() > ticket.getEndTime().getTime()){
+                if (new Date().getTime() > ticket.getEndTime().getTime()) {
                     client = null;
                     CredentialsProviderUtil.ticketExpireTime80 = 0;
                     newClient();
@@ -211,6 +214,10 @@ public class ElasticSearchAuditDestination extends AuditDestination {
                         .map(x -> new HttpHost(x, port, protocol))
                         .<HttpHost>toArray(i -> new HttpHost[i])
         );
+        ThreadFactory clientThreadFactory = new ThreadFactoryBuilder()
+                .setNameFormat("ElasticSearch rest client %s")
+                .setDaemon(true)
+                .build();
         if (StringUtils.isNotBlank(user) && StringUtils.isNotBlank(password) && !user.equalsIgnoreCase("NONE") && !password.equalsIgnoreCase("NONE")) {
             if (password.contains("keytab") && new File(password).exists()) {
                 final KerberosCredentialsProvider credentialsProvider =
@@ -218,6 +225,7 @@ public class ElasticSearchAuditDestination extends AuditDestination {
                 Lookup<AuthSchemeProvider> authSchemeRegistry = RegistryBuilder.<AuthSchemeProvider>create()
                         .register(AuthSchemes.SPNEGO, new SPNegoSchemeFactory()).build();
                 restClientBuilder.setHttpClientConfigCallback(clientBuilder -> {
+                    clientBuilder.setThreadFactory(clientThreadFactory);
                     clientBuilder.setDefaultCredentialsProvider(credentialsProvider);
                     clientBuilder.setDefaultAuthSchemeRegistry(authSchemeRegistry);
                     return clientBuilder;
@@ -225,14 +233,20 @@ public class ElasticSearchAuditDestination extends AuditDestination {
             } else {
                 final CredentialsProvider credentialsProvider =
                         CredentialsProviderUtil.getBasicCredentials(user, password);
-                restClientBuilder.setHttpClientConfigCallback(clientBuilder ->
-                        clientBuilder.setDefaultCredentialsProvider(credentialsProvider));
+                restClientBuilder.setHttpClientConfigCallback(clientBuilder -> {
+                    clientBuilder.setThreadFactory(clientThreadFactory);
+                    clientBuilder.setDefaultCredentialsProvider(credentialsProvider);
+                    return clientBuilder;
+                });
             }
         } else {
             LOG.error("ElasticSearch Credentials not provided!!");
             final CredentialsProvider credentialsProvider = null;
-            restClientBuilder.setHttpClientConfigCallback(clientBuilder ->
-                    clientBuilder.setDefaultCredentialsProvider(credentialsProvider));
+            restClientBuilder.setHttpClientConfigCallback(clientBuilder -> {
+                clientBuilder.setThreadFactory(clientThreadFactory);
+                clientBuilder.setDefaultCredentialsProvider(credentialsProvider);
+                return clientBuilder;
+            });
         }
         return restClientBuilder;
     }
@@ -254,7 +268,7 @@ public class ElasticSearchAuditDestination extends AuditDestination {
             } catch (Exception e) {
                 LOG.warn("Error validating index " + this.index);
             }
-            if(exits) {
+            if (exits) {
                 if (LOG.isDebugEnabled()) {
                     LOG.debug("Index exists");
                 }
