@@ -30,6 +30,7 @@ import javax.xml.bind.annotation.XmlAccessorType;
 import javax.xml.bind.annotation.XmlRootElement;
 
 import org.apache.commons.collections.CollectionUtils;
+import org.apache.ranger.authorization.utils.StringUtil;
 import org.codehaus.jackson.annotate.JsonAutoDetect;
 import org.codehaus.jackson.annotate.JsonIgnoreProperties;
 import org.codehaus.jackson.annotate.JsonAutoDetect.Visibility;
@@ -37,7 +38,7 @@ import org.codehaus.jackson.map.annotate.JsonSerialize;
 
 
 @JsonAutoDetect(fieldVisibility=Visibility.ANY)
-@JsonSerialize(include=JsonSerialize.Inclusion.NON_NULL)
+@JsonSerialize(include=JsonSerialize.Inclusion.NON_EMPTY)
 @JsonIgnoreProperties(ignoreUnknown=true)
 @XmlRootElement
 @XmlAccessorType(XmlAccessType.FIELD)
@@ -45,6 +46,7 @@ public class RangerPolicy extends RangerBaseModelObject implements java.io.Seria
 	public static final int POLICY_TYPE_ACCESS    = 0;
 	public static final int POLICY_TYPE_DATAMASK  = 1;
 	public static final int POLICY_TYPE_ROWFILTER = 2;
+	public static final int POLICY_TYPE_AUDIT     = 3;
 
 	public static final int[] POLICY_TYPES = new int[] {
 			POLICY_TYPE_ACCESS,
@@ -67,27 +69,28 @@ public class RangerPolicy extends RangerBaseModelObject implements java.io.Seria
 	// For future use
 	private static final long serialVersionUID = 1L;
 
-	private String                            service;
-	private String                            name;
-	private Integer                           policyType;
-	private Integer                           policyPriority;
-	private String                            description;
-	private String							  resourceSignature;
-	private Boolean                           isAuditEnabled;
-	private Map<String, RangerPolicyResource> resources;
-	private List<RangerPolicyItemCondition>   conditions;
-	private List<RangerPolicyItem>            policyItems;
-	private List<RangerPolicyItem>            denyPolicyItems;
-	private List<RangerPolicyItem>            allowExceptions;
-	private List<RangerPolicyItem>            denyExceptions;
-	private List<RangerDataMaskPolicyItem>    dataMaskPolicyItems;
-	private List<RangerRowFilterPolicyItem>   rowFilterPolicyItems;
-	private String                            serviceType;
-	private Map<String, Object>               options;
-	private List<RangerValiditySchedule>      validitySchedules;
-	private List<String>                      policyLabels;
-	private String                            zoneName;
-	private Boolean                           isDenyAllElse;
+	private String                                  service;
+	private String                                  name;
+	private Integer                                 policyType;
+	private Integer                                 policyPriority;
+	private String                                  description;
+	private String                                  resourceSignature;
+	private Boolean                                 isAuditEnabled;
+	private Map<String, RangerPolicyResource>       resources;
+	private List<Map<String, RangerPolicyResource>> additionalResources;
+	private List<RangerPolicyItemCondition>         conditions;
+	private List<RangerPolicyItem>                  policyItems;
+	private List<RangerPolicyItem>                  denyPolicyItems;
+	private List<RangerPolicyItem>                  allowExceptions;
+	private List<RangerPolicyItem>                  denyExceptions;
+	private List<RangerDataMaskPolicyItem>          dataMaskPolicyItems;
+	private List<RangerRowFilterPolicyItem>         rowFilterPolicyItems;
+	private String                                  serviceType;
+	private Map<String, Object>                     options;
+	private List<RangerValiditySchedule>            validitySchedules;
+	private List<String>                            policyLabels;
+	private String                                  zoneName;
+	private Boolean                                 isDenyAllElse;
 
 	public RangerPolicy() {
 		this(null, null, null, null, null, null, null, null, null, null, null);
@@ -154,17 +157,19 @@ public class RangerPolicy extends RangerBaseModelObject implements java.io.Seria
 		setResourceSignature(other.getResourceSignature());
 		setIsAuditEnabled(other.getIsAuditEnabled());
 		setResources(other.getResources());
+		setAdditionalResources(other.getAdditionalResources());
+		setConditions(other.getConditions());
 		setPolicyItems(other.getPolicyItems());
 		setDenyPolicyItems(other.getDenyPolicyItems());
 		setAllowExceptions(other.getAllowExceptions());
 		setDenyExceptions(other.getDenyExceptions());
 		setDataMaskPolicyItems(other.getDataMaskPolicyItems());
 		setRowFilterPolicyItems(other.getRowFilterPolicyItems());
+		setServiceType(other.getServiceType());
 		setOptions(other.getOptions());
 		setValiditySchedules(other.getValiditySchedules());
 		setPolicyLabels(other.getPolicyLabels());
 		setZoneName(other.getZoneName());
-		setConditions(other.getConditions());
 		setIsDenyAllElse(other.getIsDenyAllElse());
 	}
 
@@ -322,6 +327,28 @@ public class RangerPolicy extends RangerBaseModelObject implements java.io.Seria
 		}
 	}
 
+	public List<Map<String, RangerPolicyResource>> getAdditionalResources() {
+		return additionalResources;
+	}
+
+	public void setAdditionalResources(List<Map<String, RangerPolicyResource>> additionalResources) {
+		this.additionalResources = additionalResources;
+	}
+
+	public void addResource(Map<String, RangerPolicyResource> resources) {
+		if (resources != null && !resources.isEmpty()) {
+			if (this.resources == null || this.resources.isEmpty()) {
+				this.resources = resources;
+			} else {
+				if (this.additionalResources == null) {
+					this.additionalResources = new ArrayList<>();
+				}
+
+				this.additionalResources.add(resources);
+			}
+		}
+	}
+
 	/**
 	 * @return the policyItems
 	 */
@@ -524,6 +551,71 @@ public class RangerPolicy extends RangerBaseModelObject implements java.io.Seria
 		this.isDenyAllElse = isDenyAllElse == null ? Boolean.FALSE : isDenyAllElse;
 	}
 
+	public void dedupStrings(Map<String, String> strTbl) {
+		super.dedupStrings(strTbl);
+
+		service      = StringUtil.dedupString(service, strTbl);
+		serviceType  = StringUtil.dedupString(serviceType, strTbl);
+		zoneName     = StringUtil.dedupString(zoneName, strTbl);
+		name         = StringUtil.dedupString(name, strTbl);
+		description  = StringUtil.dedupString(description, strTbl);
+		policyLabels = StringUtil.dedupStringsList(policyLabels, strTbl);
+		resources    = StringUtil.dedupStringsMapOfPolicyResource(resources, strTbl);
+		options      = StringUtil.dedupStringsMapOfObject(options, strTbl);
+
+		if (CollectionUtils.isNotEmpty(additionalResources)) {
+			List<Map<String, RangerPolicyResource>> updated = new ArrayList<>(additionalResources.size());
+
+			for (Map<String, RangerPolicyResource> additionalResource : additionalResources) {
+				updated.add(StringUtil.dedupStringsMapOfPolicyResource(additionalResource, strTbl));
+			}
+
+			additionalResources = updated;
+		}
+
+		if (CollectionUtils.isNotEmpty(conditions)) {
+			for (RangerPolicyItemCondition condition : conditions) {
+				condition.dedupStrings(strTbl);
+			}
+		}
+
+		if (CollectionUtils.isNotEmpty(policyItems)) {
+			for (RangerPolicyItem policyItem : policyItems) {
+				policyItem.dedupStrings(strTbl);
+			}
+		}
+
+		if (CollectionUtils.isNotEmpty(denyPolicyItems)) {
+			for (RangerPolicyItem policyItem : denyPolicyItems) {
+				policyItem.dedupStrings(strTbl);
+			}
+		}
+
+		if (CollectionUtils.isNotEmpty(allowExceptions)) {
+			for (RangerPolicyItem policyItem : allowExceptions) {
+				policyItem.dedupStrings(strTbl);
+			}
+		}
+
+		if (CollectionUtils.isNotEmpty(denyExceptions)) {
+			for (RangerPolicyItem policyItem : denyExceptions) {
+				policyItem.dedupStrings(strTbl);
+			}
+		}
+
+		if (CollectionUtils.isNotEmpty(dataMaskPolicyItems)) {
+			for (RangerPolicyItem policyItem : dataMaskPolicyItems) {
+				policyItem.dedupStrings(strTbl);
+			}
+		}
+
+		if (CollectionUtils.isNotEmpty(rowFilterPolicyItems)) {
+			for (RangerPolicyItem policyItem : rowFilterPolicyItems) {
+				policyItem.dedupStrings(strTbl);
+			}
+		}
+	}
+
 	@Override
 	public String toString( ) {
 		StringBuilder sb = new StringBuilder();
@@ -556,6 +648,19 @@ public class RangerPolicy extends RangerBaseModelObject implements java.io.Seria
 			}
 		}
 		sb.append("} ");
+		sb.append("additionalResources={");
+		if(additionalResources != null) {
+			for(Map<String, RangerPolicyResource> additionalResource : additionalResources) {
+				sb.append("{");
+				for(Map.Entry<String, RangerPolicyResource> e : additionalResource.entrySet()) {
+					sb.append(e.getKey()).append("={");
+					e.getValue().toString(sb);
+					sb.append("} ");
+				}
+				sb.append("} ");
+			}
+		}
+		sb.append("} ");
                 sb.append("policyLabels={");
                 if(policyLabels != null) {
                         for(String policyLabel : policyLabels) {
@@ -648,7 +753,7 @@ public class RangerPolicy extends RangerBaseModelObject implements java.io.Seria
 
         //sb.append("validitySchedules={").append(validitySchedules).append("} ");
         sb.append("validitySchedules={");
-        if (CollectionUtils.isNotEmpty(validitySchedules)) {
+        if (validitySchedules != null) {
             for (RangerValiditySchedule schedule : validitySchedules) {
                 if (schedule != null) {
                     sb.append("schedule={").append(schedule).append("}");
@@ -674,7 +779,7 @@ public class RangerPolicy extends RangerBaseModelObject implements java.io.Seria
 	}
 
 	@JsonAutoDetect(fieldVisibility=Visibility.ANY)
-	@JsonSerialize(include=JsonSerialize.Inclusion.NON_NULL)
+	@JsonSerialize(include=JsonSerialize.Inclusion.NON_EMPTY)
 	@JsonIgnoreProperties(ignoreUnknown=true)
 	@XmlRootElement
 	@XmlAccessorType(XmlAccessType.FIELD)
@@ -774,6 +879,10 @@ public class RangerPolicy extends RangerBaseModelObject implements java.io.Seria
 			this.isRecursive = isRecursive == null ? Boolean.FALSE : isRecursive;
 		}
 
+		public void dedupStrings(Map<String, String> strTbl) {
+			values = StringUtil.dedupStringsList(values, strTbl);
+		}
+
 		@Override
 		public String toString( ) {
 			StringBuilder sb = new StringBuilder();
@@ -842,7 +951,7 @@ public class RangerPolicy extends RangerBaseModelObject implements java.io.Seria
 	}
 
 	@JsonAutoDetect(fieldVisibility=Visibility.ANY)
-	@JsonSerialize(include=JsonSerialize.Inclusion.NON_NULL)
+	@JsonSerialize(include=JsonSerialize.Inclusion.NON_EMPTY)
 	@JsonIgnoreProperties(ignoreUnknown=true)
 	@XmlRootElement
 	@XmlAccessorType(XmlAccessType.FIELD)
@@ -1004,6 +1113,24 @@ public class RangerPolicy extends RangerBaseModelObject implements java.io.Seria
 			this.delegateAdmin = delegateAdmin == null ? Boolean.FALSE : delegateAdmin;
 		}
 
+		public void dedupStrings(Map<String, String> strTbl) {
+			if (accesses != null) {
+				for (RangerPolicyItemAccess access : accesses) {
+					access.dedupStrings(strTbl);
+				}
+			}
+
+			users  = StringUtil.dedupStringsList(users, strTbl);
+			groups = StringUtil.dedupStringsList(groups, strTbl);
+			roles  = StringUtil.dedupStringsList(roles, strTbl);
+
+			if (conditions != null) {
+				for (RangerPolicyItemCondition condition : conditions) {
+					condition.dedupStrings(strTbl);
+				}
+			}
+		}
+
 		@Override
 		public String toString( ) {
 			StringBuilder sb = new StringBuilder();
@@ -1135,7 +1262,7 @@ public class RangerPolicy extends RangerBaseModelObject implements java.io.Seria
 	}
 
 	@JsonAutoDetect(fieldVisibility=Visibility.ANY)
-	@JsonSerialize(include=JsonSerialize.Inclusion.NON_NULL)
+	@JsonSerialize(include=JsonSerialize.Inclusion.NON_EMPTY)
 	@JsonIgnoreProperties(ignoreUnknown=true)
 	@XmlRootElement
 	@XmlAccessorType(XmlAccessType.FIELD)
@@ -1222,7 +1349,7 @@ public class RangerPolicy extends RangerBaseModelObject implements java.io.Seria
 	}
 
 	@JsonAutoDetect(fieldVisibility=Visibility.ANY)
-	@JsonSerialize(include=JsonSerialize.Inclusion.NON_NULL)
+	@JsonSerialize(include=JsonSerialize.Inclusion.NON_EMPTY)
 	@JsonIgnoreProperties(ignoreUnknown=true)
 	@XmlRootElement
 	@XmlAccessorType(XmlAccessType.FIELD)
@@ -1309,7 +1436,7 @@ public class RangerPolicy extends RangerBaseModelObject implements java.io.Seria
 	}
 
 	@JsonAutoDetect(fieldVisibility=Visibility.ANY)
-	@JsonSerialize(include=JsonSerialize.Inclusion.NON_NULL)
+	@JsonSerialize(include=JsonSerialize.Inclusion.NON_EMPTY)
 	@JsonIgnoreProperties(ignoreUnknown=true)
 	@XmlRootElement
 	@XmlAccessorType(XmlAccessType.FIELD)
@@ -1343,7 +1470,7 @@ public class RangerPolicy extends RangerBaseModelObject implements java.io.Seria
 		 * @param type the type to set
 		 */
 		public void setType(String type) {
-			this.type = type;
+			this.type = type == null ? "" : type;
 		}
 
 		/**
@@ -1360,6 +1487,10 @@ public class RangerPolicy extends RangerBaseModelObject implements java.io.Seria
 			this.isAllowed = isAllowed == null ? Boolean.TRUE : isAllowed;
 		}
 
+		public void dedupStrings(Map<String, String> strTbl) {
+			type = StringUtil.dedupString(type, strTbl);
+		}
+
 		@Override
 		public String toString( ) {
 			StringBuilder sb = new StringBuilder();
@@ -1413,7 +1544,7 @@ public class RangerPolicy extends RangerBaseModelObject implements java.io.Seria
 	}
 
 	@JsonAutoDetect(fieldVisibility=Visibility.ANY)
-	@JsonSerialize(include=JsonSerialize.Inclusion.NON_NULL)
+	@JsonSerialize(include=JsonSerialize.Inclusion.NON_EMPTY)
 	@JsonIgnoreProperties(ignoreUnknown=true)
 	@XmlRootElement
 	@XmlAccessorType(XmlAccessType.FIELD)
@@ -1472,6 +1603,11 @@ public class RangerPolicy extends RangerBaseModelObject implements java.io.Seria
 			}
 		}
 
+		public void dedupStrings(Map<String, String> strTbl) {
+			type   = StringUtil.dedupString(type, strTbl);
+			values = StringUtil.dedupStringsList(values, strTbl);
+		}
+
 		@Override
 		public String toString( ) {
 			StringBuilder sb = new StringBuilder();
@@ -1531,7 +1667,7 @@ public class RangerPolicy extends RangerBaseModelObject implements java.io.Seria
 	}
 
 	@JsonAutoDetect(fieldVisibility=Visibility.ANY)
-	@JsonSerialize(include=JsonSerialize.Inclusion.NON_NULL)
+	@JsonSerialize(include=JsonSerialize.Inclusion.NON_EMPTY)
 	@JsonIgnoreProperties(ignoreUnknown=true)
 	@XmlRootElement
 	@XmlAccessorType(XmlAccessType.FIELD)
@@ -1550,6 +1686,12 @@ public class RangerPolicy extends RangerBaseModelObject implements java.io.Seria
 			setValueExpr(valueExpr);
 		}
 
+		public RangerPolicyItemDataMaskInfo(RangerPolicyItemDataMaskInfo that) {
+			this.dataMaskType  = that.dataMaskType;
+			this.conditionExpr = that.conditionExpr;
+			this.valueExpr     = that.valueExpr;
+		}
+
 		public String getDataMaskType() {
 			return dataMaskType;
 		}
@@ -1586,8 +1728,6 @@ public class RangerPolicy extends RangerBaseModelObject implements java.io.Seria
 
 		@Override
 		public boolean equals(Object obj) {
-			if(! super.equals(obj))
-				return false;
 			if (this == obj)
 				return true;
 			if (obj == null)
@@ -1636,7 +1776,7 @@ public class RangerPolicy extends RangerBaseModelObject implements java.io.Seria
 	}
 
 	@JsonAutoDetect(fieldVisibility=Visibility.ANY)
-	@JsonSerialize(include=JsonSerialize.Inclusion.NON_NULL)
+	@JsonSerialize(include=JsonSerialize.Inclusion.NON_EMPTY)
 	@JsonIgnoreProperties(ignoreUnknown=true)
 	@XmlRootElement
 	@XmlAccessorType(XmlAccessType.FIELD)
@@ -1651,6 +1791,10 @@ public class RangerPolicy extends RangerBaseModelObject implements java.io.Seria
 			setFilterExpr(filterExpr);
 		}
 
+		public RangerPolicyItemRowFilterInfo(RangerPolicyItemRowFilterInfo that) {
+			this.filterExpr = that.filterExpr;
+		}
+
 		public String getFilterExpr() {
 			return filterExpr;
 		}
@@ -1669,8 +1813,6 @@ public class RangerPolicy extends RangerBaseModelObject implements java.io.Seria
 
 		@Override
 		public boolean equals(Object obj) {
-			if(! super.equals(obj))
-				return false;
 			if (this == obj)
 				return true;
 			if (obj == null)
@@ -1696,7 +1838,7 @@ public class RangerPolicy extends RangerBaseModelObject implements java.io.Seria
 		}
 
 		public StringBuilder toString(StringBuilder sb) {
-			sb.append("RangerPolicyItemDataMaskInfo={");
+			sb.append("RangerPolicyItemRowFilterInfo={");
 
 			sb.append("filterExpr={").append(filterExpr).append("} ");
 
