@@ -19,6 +19,7 @@
 
 package org.apache.ranger.plugin.model;
 
+import org.apache.ranger.authorization.utils.StringUtil;
 import org.codehaus.jackson.annotate.JsonAutoDetect;
 import org.codehaus.jackson.annotate.JsonIgnoreProperties;
 import org.codehaus.jackson.map.annotate.JsonSerialize;
@@ -26,11 +27,10 @@ import org.codehaus.jackson.map.annotate.JsonSerialize;
 import javax.xml.bind.annotation.XmlAccessType;
 import javax.xml.bind.annotation.XmlAccessorType;
 import javax.xml.bind.annotation.XmlRootElement;
-import java.util.HashMap;
 import java.util.Map;
 
 @JsonAutoDetect(fieldVisibility=JsonAutoDetect.Visibility.ANY)
-@JsonSerialize(include=JsonSerialize.Inclusion.NON_NULL)
+@JsonSerialize(include=JsonSerialize.Inclusion.NON_EMPTY)
 @JsonIgnoreProperties(ignoreUnknown=true)
 @XmlRootElement
 @XmlAccessorType(XmlAccessType.FIELD)
@@ -92,7 +92,7 @@ public class RangerServiceResource extends RangerBaseModelObject {
 	}
 
 	public void setResourceElements(Map<String, RangerPolicy.RangerPolicyResource> resource) {
-		this.resourceElements = resource == null ? new HashMap<String, RangerPolicy.RangerPolicyResource>() : resource;
+		this.resourceElements = resource;
 	}
 
 	public void setResourceSignature(String resourceSignature) {
@@ -107,6 +107,15 @@ public class RangerServiceResource extends RangerBaseModelObject {
 		this.additionalInfo = additionalInfo;
 	}
 
+	public void dedupStrings(Map<String, String> strTbl) {
+		super.dedupStrings(strTbl);
+
+		serviceName      = StringUtil.dedupString(serviceName, strTbl);
+		resourceElements = StringUtil.dedupStringsMapOfPolicyResource(resourceElements, strTbl);
+		ownerUser        = StringUtil.dedupString(ownerUser, strTbl);
+		additionalInfo   = StringUtil.dedupStringsMap(additionalInfo, strTbl);
+	}
+
 	@Override
 	public String toString() {
 		StringBuilder sb = new StringBuilder();
