@@ -29,10 +29,12 @@ import java.util.Set;
 
 import org.apache.commons.collections.CollectionUtils;
 import org.apache.commons.lang.StringUtils;
-import org.apache.log4j.Logger;
+import org.apache.ranger.plugin.util.RangerAccessRequestUtil;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
 public class RangerAccessRequestImpl implements RangerAccessRequest {
-	private static final Logger LOG = Logger.getLogger(RangerAccessRequestImpl.class);
+	private static final Logger LOG = LoggerFactory.getLogger(RangerAccessRequestImpl.class);
 
 	private RangerAccessResource resource;
 	private String               accessType;
@@ -78,6 +80,26 @@ public class RangerAccessRequestImpl implements RangerAccessRequest {
 		setClusterName(null);
 	}
 
+	public RangerAccessRequestImpl(RangerAccessRequest request) {
+		setResource(request.getResource());
+		setAccessType(request.getAccessType());
+		setUser(request.getUser());
+		setUserGroups(request.getUserGroups());
+		setUserRoles(request.getUserRoles());
+		setForwardedAddresses(request.getForwardedAddresses());
+		setAccessTime(request.getAccessTime());
+		setRemoteIPAddress(request.getRemoteIPAddress());
+		setClientType(request.getClientType());
+		setAction(request.getAction());
+		setRequestData(request.getRequestData());
+		setSessionId(request.getSessionId());
+		setContext(request.getContext());
+		setClusterName(request.getClusterName());
+		setResourceMatchingScope(request.getResourceMatchingScope());
+		setClientIPAddress(request.getClientIPAddress());
+		setClusterType(request.getClusterType());
+	}
+
 	@Override
 	public RangerAccessResource getResource() {
 		return resource;
@@ -236,7 +258,17 @@ public class RangerAccessRequestImpl implements RangerAccessRequest {
 	public void setResourceMatchingScope(ResourceMatchingScope scope) { this.resourceMatchingScope = scope; }
 
 	public void setContext(Map<String, Object> context) {
-		this.context = (context == null) ? new HashMap<String, Object>() : context;
+		if (context == null) {
+			this.context = new HashMap<>();
+		} else {
+			this.context = context;
+		}
+
+		RangerAccessRequest current = RangerAccessRequestUtil.getRequestFromContext(this.context);
+
+		if (current == null) {
+			RangerAccessRequestUtil.setRequestInContext(this);
+		}
 	}
 
 	public void extractAndSetClientIPAddress(boolean useForwardedIPAddress, String[]trustedProxyAddresses) {
@@ -324,7 +356,11 @@ public class RangerAccessRequestImpl implements RangerAccessRequest {
 		sb.append("context={");
 		if(context != null) {
 			for(Map.Entry<String, Object> e : context.entrySet()) {
-				sb.append(e.getKey()).append("={").append(e.getValue()).append("} ");
+				Object val = e.getValue();
+
+				if (!(val instanceof RangerAccessRequest)) { // to avoid recursive calls
+					sb.append(e.getKey()).append("={").append(val).append("} ");
+				}
 			}
 		}
 		sb.append("} ");
