@@ -34,7 +34,9 @@ import org.apache.ranger.plugin.model.RangerServiceDef;
 import org.apache.ranger.plugin.policyengine.RangerResourceACLs.DataMaskResult;
 import org.apache.ranger.plugin.policyengine.RangerResourceACLs.RowFilterResult;
 import org.apache.ranger.plugin.policyevaluator.RangerPolicyEvaluator;
+import org.apache.ranger.plugin.policyevaluator.RangerPolicyEvaluator.RangerPolicyResourceEvaluator;
 import org.apache.ranger.plugin.policyevaluator.RangerPolicyEvaluator.PolicyACLSummary;
+import org.apache.ranger.plugin.policyresourcematcher.RangerPolicyResourceMatcher;
 import org.apache.ranger.plugin.policyresourcematcher.RangerPolicyResourceMatcher.MatchType;
 import org.apache.ranger.plugin.service.RangerDefaultRequestProcessor;
 import org.apache.ranger.plugin.util.GrantRevokeRequest;
@@ -307,30 +309,46 @@ public class RangerPolicyEngineImpl implements RangerPolicyEngine {
 						policyPriority = evaluator.getPolicyPriority();
 					}
 
-					MatchType matchType = tagMatchTypeMap.get(evaluator.getId());
+					MatchType matchType = tagMatchTypeMap.get(evaluator.getPolicyId());
+
+					boolean isMatched = false;
+					boolean isConditionalMatch = false;
 
 					if (matchType == null) {
-						matchType = evaluator.getPolicyResourceMatcher().getMatchType(request.getResource(), request.getContext());
-					}
+						for (RangerPolicyResourceEvaluator resourceEvaluator : evaluator.getResourceEvaluators()) {
+							RangerPolicyResourceMatcher matcher = resourceEvaluator.getPolicyResourceMatcher();
 
-					final boolean isMatched;
+							matchType = matcher.getMatchType(request.getResource(), request.getContext());
+							isMatched = isMatch(matchType, request.getResourceMatchingScope());
 
-					if (request.getResourceMatchingScope() == RangerAccessRequest.ResourceMatchingScope.SELF_OR_DESCENDANTS) {
-						isMatched = matchType != MatchType.NONE;
+							if (isMatched) {
+								isConditionalMatch = false;
+
+								break;
+							} else if (matcher.getNeedsDynamicEval() && !isConditionalMatch) {
+								MatchType dynWildCardMatch = resourceEvaluator.getMacrosReplaceWithWildcardMatcher(policyEngine).getMatchType(request.getResource(), request.getContext());
+
+								isConditionalMatch = isMatch(dynWildCardMatch, request.getResourceMatchingScope());
+							}
+						}
 					} else {
-						isMatched = matchType == MatchType.SELF || matchType == MatchType.SELF_AND_ALL_DESCENDANTS;
+						isMatched = isMatch(matchType, request.getResourceMatchingScope());
 					}
 
-					if (!isMatched) {
+					if (!isMatched && !isConditionalMatch) {
 						continue;
 					}
 
+					if (!isConditionalMatch) {
+						isConditionalMatch = policyIdForTemporalTags.contains(evaluator.getPolicyId()) || evaluator.getValidityScheduleEvaluatorsCount() != 0;
+					}
+
 					if (policyType == RangerPolicy.POLICY_TYPE_ACCESS) {
-						updateFromPolicyACLs(evaluator, policyIdForTemporalTags, ret);
+						updateFromPolicyACLs(evaluator, isConditionalMatch, ret);
 					} else if (policyType == RangerPolicy.POLICY_TYPE_ROWFILTER) {
-						updateRowFiltersFromPolicy(evaluator, policyIdForTemporalTags, ret);
+						updateRowFiltersFromPolicy(evaluator, isConditionalMatch, ret);
 					} else if (policyType == RangerPolicy.POLICY_TYPE_DATAMASK) {
-						updateDataMasksFromPolicy(evaluator, policyIdForTemporalTags, ret);
+						updateDataMasksFromPolicy(evaluator, isConditionalMatch, ret);
 					}
 				}
 
@@ -1009,10 +1027,10 @@ public class RangerPolicyEngineImpl implements RangerPolicyEngine {
 					RangerTagForEval tag = tagEvaluator.getTag();
 
 					allEvaluators.add(evaluator);
-					tagMatchTypeMap.put(evaluator.getId(), tag.getMatchType());
+					tagMatchTypeMap.put(evaluator.getPolicyId(), tag.getMatchType());
 
 					if (CollectionUtils.isNotEmpty(tag.getValidityPeriods())) {
-						policyIdForTemporalTags.add(evaluator.getId());
+						policyIdForTemporalTags.add(evaluator.getPolicyId());
 					}
 				}
 			}
@@ -1159,15 +1177,13 @@ public class RangerPolicyEngineImpl implements RangerPolicyEngine {
 		return policyEngine.getPluginContext().getConfig().getIsFallbackSupported();
 	}
 
-	private void updateFromPolicyACLs(RangerPolicyEvaluator evaluator, Set<Long> policyIdForTemporalTags, RangerResourceACLs resourceACLs) {
+	private void updateFromPolicyACLs(RangerPolicyEvaluator evaluator, boolean isConditional, RangerResourceACLs resourceACLs) {
 		PolicyACLSummary aclSummary = evaluator.getPolicyACLSummary();
 
 		if (aclSummary == null) {
 			return;
 		}
 
-		boolean isConditional = policyIdForTemporalTags.contains(evaluator.getId()) || evaluator.getValidityScheduleEvaluatorsCount() != 0;
-
 		for (Map.Entry<String, Map<String, PolicyACLSummary.AccessResult>> userAccessInfo : aclSummary.getUsersAccessInfo().entrySet()) {
 			final String userName = userAccessInfo.getKey();
 
@@ -1235,12 +1251,10 @@ public class RangerPolicyEngineImpl implements RangerPolicyEngine {
 		}
 	}
 
-	private void updateRowFiltersFromPolicy(RangerPolicyEvaluator evaluator, Set<Long> policyIdForTemporalTags, RangerResourceACLs resourceACLs) {
+	private void updateRowFiltersFromPolicy(RangerPolicyEvaluator evaluator, boolean isConditional, RangerResourceACLs resourceACLs) {
 		PolicyACLSummary aclSummary = evaluator.getPolicyACLSummary();
 
 		if (aclSummary != null) {
-			boolean isConditional = policyIdForTemporalTags.contains(evaluator.getId()) || evaluator.getValidityScheduleEvaluatorsCount() != 0;
-
 			for (RowFilterResult rowFilterResult : aclSummary.getRowFilters()) {
 				rowFilterResult = copyRowFilter(rowFilterResult);
 
@@ -1253,12 +1267,10 @@ public class RangerPolicyEngineImpl implements RangerPolicyEngine {
 		}
 	}
 
-	private void updateDataMasksFromPolicy(RangerPolicyEvaluator evaluator, Set<Long> policyIdForTemporalTags, RangerResourceACLs resourceACLs) {
+	private void updateDataMasksFromPolicy(RangerPolicyEvaluator evaluator, boolean isConditional, RangerResourceACLs resourceACLs) {
 		PolicyACLSummary aclSummary = evaluator.getPolicyACLSummary();
 
 		if (aclSummary != null) {
-			boolean isConditional = policyIdForTemporalTags.contains(evaluator.getId()) || evaluator.getValidityScheduleEvaluatorsCount() != 0;
-
 			for (DataMaskResult dataMaskResult : aclSummary.getDataMasks()) {
 				dataMaskResult = copyDataMask(dataMaskResult);
 
@@ -1299,6 +1311,18 @@ public class RangerPolicyEngineImpl implements RangerPolicyEngine {
 		return values != null ? new HashSet<>(values) : null;
 	}
 
+	private boolean isMatch(MatchType matchType, RangerAccessRequest.ResourceMatchingScope matchingScope) {
+		final boolean ret;
+
+		if (matchingScope == RangerAccessRequest.ResourceMatchingScope.SELF_OR_DESCENDANTS) {
+			ret = matchType != MatchType.NONE;
+		} else {
+			ret = matchType == MatchType.SELF || matchType == MatchType.SELF_AND_ALL_DESCENDANTS;
+		}
+
+		return ret;
+	}
+
 	private static class ServiceConfig {
 		private final Set<String> auditExcludedUsers;
 		private final Set<String> auditExcludedGroups;
