@@ -44,10 +44,12 @@ import org.apache.ranger.plugin.model.RangerServiceDef;
 import org.apache.ranger.plugin.model.RangerServiceResource;
 import org.apache.ranger.plugin.model.RangerValiditySchedule;
 import org.apache.ranger.plugin.model.validation.RangerValidityScheduleValidator;
+import org.apache.ranger.plugin.model.validation.RangerZoneResourceMatcher;
 import org.apache.ranger.plugin.model.validation.ValidationFailureDetails;
 import org.apache.ranger.plugin.policyengine.TestPolicyEngine.PolicyEngineTestCase.TestData;
+import org.apache.ranger.plugin.policyevaluator.RangerPolicyEvaluator.RangerPolicyResourceEvaluator;
 import org.apache.ranger.plugin.policyevaluator.RangerValidityScheduleEvaluator;
-import org.apache.ranger.plugin.policyresourcematcher.RangerPolicyResourceEvaluator;
+import org.apache.ranger.plugin.policyresourcematcher.RangerResourceEvaluator;
 import org.apache.ranger.plugin.service.RangerBasePlugin;
 import org.apache.ranger.plugin.util.RangerAccessRequestUtil;
 import org.apache.ranger.plugin.util.RangerRequestedResources;
@@ -348,6 +350,13 @@ public class TestPolicyEngine {
 		runTestsFromResourceFiles(resourceFiles);
 	}
 
+	@Test
+	public void testPolicyEngine_hiveMaskingWithReqExpressions() {
+		String[] resourceFiles = {"/policyengine/test_policyengine_hive_mask_filter_with_req_expressions.json"};
+
+		runTestsFromResourceFiles(resourceFiles);
+	}
+
 	@Test
 	public void testPolicyEngine_hiveTagMasking() {
 		String[] resourceFiles = {"/policyengine/test_policyengine_tag_hive_mask.json"};
@@ -465,6 +474,13 @@ public class TestPolicyEngine {
 		runTestsFromResourceFiles(resourceFiles);
 	}
 
+	@Test
+	public void testPolicyEngin_policyWithAdditionalResources() {
+		String[] resourceFiles = {"/policyengine/test_policyengine_policy_with_additional_resources.json"};
+
+		runTestsFromResourceFiles(resourceFiles);
+	}
+
 	private void runTestsFromResourceFiles(String[] resourceNames) {
 		for(String resourceName : resourceNames) {
 			InputStream inStream = this.getClass().getResourceAsStream(resourceName);
@@ -504,8 +520,9 @@ public class TestPolicyEngine {
 			servicePolicies.setTagPolicies(tagPolicies);
 		}
 
-		boolean useForwardedIPAddress = pluginContext.getConfig().getBoolean("ranger.plugin.hive.use.x-forwarded-for.ipaddress", false);
-		String trustedProxyAddressString = pluginContext.getConfig().get("ranger.plugin.hive.trusted.proxy.ipaddresses");
+		RangerPluginConfig config = pluginContext.getConfig();
+		boolean useForwardedIPAddress = config.getBoolean(config.getPropertyPrefix() + ".use.x-forwarded-for.ipaddress", false);
+		String trustedProxyAddressString = config.get(config.getPropertyPrefix() + ".trusted.proxy.ipaddresses");
 		String[] trustedProxyAddresses = StringUtils.split(trustedProxyAddressString, ';');
 		if (trustedProxyAddresses != null) {
 			for (int i = 0; i < trustedProxyAddresses.length; i++) {
@@ -562,18 +579,18 @@ public class TestPolicyEngine {
 
 		roles.setRangerRoles(rolesSet);
 
-        RangerPolicyEngineOptions policyEngineOptions = pluginContext.getConfig().getPolicyEngineOptions();
+        RangerPolicyEngineOptions policyEngineOptions = config.getPolicyEngineOptions();
 
         policyEngineOptions.disableAccessEvaluationWithPolicyACLSummary = true;
 
-        setPluginConfig(pluginContext.getConfig(), ".super.users", testCase.superUsers);
-        setPluginConfig(pluginContext.getConfig(), ".super.groups", testCase.superGroups);
-        setPluginConfig(pluginContext.getConfig(), ".audit.exclude.users", testCase.auditExcludedUsers);
-        setPluginConfig(pluginContext.getConfig(), ".audit.exclude.groups", testCase.auditExcludedGroups);
-        setPluginConfig(pluginContext.getConfig(), ".audit.exclude.roles", testCase.auditExcludedRoles);
+        setPluginConfig(config, ".super.users", testCase.superUsers);
+        setPluginConfig(config, ".super.groups", testCase.superGroups);
+        setPluginConfig(config, ".audit.exclude.users", testCase.auditExcludedUsers);
+        setPluginConfig(config, ".audit.exclude.groups", testCase.auditExcludedGroups);
+        setPluginConfig(config, ".audit.exclude.roles", testCase.auditExcludedRoles);
 
         // so that setSuperUsersAndGroups(), setAuditExcludedUsersGroupsRoles() will be called on the pluginConfig
-        new RangerBasePlugin(pluginContext.getConfig());
+        new RangerBasePlugin(config);
 
         RangerPolicyEngineImpl policyEngine = new RangerPolicyEngineImpl(servicePolicies, pluginContext, roles);
 
@@ -672,19 +689,19 @@ public class TestPolicyEngine {
 
 			RangerAccessResultProcessor auditHandler = new RangerDefaultAuditHandler();
 
+			if (MapUtils.isNotEmpty(test.userAttributes) || MapUtils.isNotEmpty(test.groupAttributes)) {
+				RangerUserStore userStore = new RangerUserStore();
+
+				userStore.setUserAttrMapping(test.userAttributes);
+				userStore.setGroupAttrMapping(test.groupAttributes);
+
+				RangerAccessRequestUtil.setRequestUserStoreInContext(request.getContext(), userStore);
+			}
+
 			if(test.result != null) {
                 RangerAccessResult expected = test.result;
                 RangerAccessResult result;
 
-                if (MapUtils.isNotEmpty(test.userAttributes) || MapUtils.isNotEmpty(test.groupAttributes)) {
-                    RangerUserStore userStore = new RangerUserStore();
-
-                    userStore.setUserAttrMapping(test.userAttributes);
-                    userStore.setGroupAttrMapping(test.groupAttributes);
-
-                    RangerAccessRequestUtil.setRequestUserStoreInContext(request.getContext(), userStore);
-                }
-
 				result   = policyEngine.evaluatePolicies(request, RangerPolicy.POLICY_TYPE_ACCESS, auditHandler);
 
 				policyEngine.evaluateAuditPolicies(result);
@@ -890,9 +907,9 @@ public class TestPolicyEngine {
                     }
                 }
 
-                assertTrue(testCase.name, isValid == testCase.result.isValid);
-                assertTrue(testCase.name, isApplicable == testCase.result.isApplicable);
-                assertTrue(testCase.name + ", [" + validationFailures +"]", validationFailures.size() == testCase.result.validationFailureCount);
+                assertEquals(testCase.name + " - isValid (validationFailures: " + validationFailures + ")", testCase.result.isValid, isValid);
+				assertEquals(testCase.name + " - isApplicable (validationFailures: " + validationFailures + ")", testCase.result.isApplicable, isApplicable);
+                assertEquals(testCase.name + " - validationFailureCount (validationFailures: " + validationFailures +")", testCase.result.validationFailureCount, validationFailures.size());
             }
         }
         TimeZone.setDefault(defaultTZ);
@@ -943,7 +960,7 @@ public class TestPolicyEngine {
 			ret = Objects.equals(me.getResourceZoneTrie().keySet(), other.getResourceZoneTrie().keySet());
 
 			if (ret) {
-				for (Map.Entry<String, RangerResourceTrie> entry : me.getResourceZoneTrie().entrySet()) {
+				for (Map.Entry<String, RangerResourceTrie<RangerZoneResourceMatcher>> entry : me.getResourceZoneTrie().entrySet()) {
 					ret = compareSubtree(entry.getValue(), other.getResourceZoneTrie().get(entry.getKey()));
 
 					if (!ret) {
@@ -979,13 +996,13 @@ public class TestPolicyEngine {
 	public static boolean compareTrie(final int policyType, RangerPolicyRepository me, RangerPolicyRepository other) {
 		boolean ret;
 
-		Map<String, RangerResourceTrie> myTrie    = me.getTrie(policyType);
-		Map<String, RangerResourceTrie> otherTrie = other.getTrie(policyType);
+		Map<String, RangerResourceTrie<RangerPolicyResourceEvaluator>> myTrie    = me.getTrie(policyType);
+		Map<String, RangerResourceTrie<RangerPolicyResourceEvaluator>> otherTrie = other.getTrie(policyType);
 
 		ret = myTrie.size() == otherTrie.size();
 
 		if (ret) {
-			for (Map.Entry<String, RangerResourceTrie> entry : myTrie.entrySet()) {
+			for (Map.Entry<String, RangerResourceTrie<RangerPolicyResourceEvaluator>> entry : myTrie.entrySet()) {
 				RangerResourceTrie myResourceTrie    = entry.getValue();
 				RangerResourceTrie otherResourceTrie = otherTrie.get(entry.getKey());
 
@@ -1108,28 +1125,36 @@ public class TestPolicyEngine {
 			ret = me.size() == other.size();
 
 			if (ret) {
-				List<? extends RangerPolicyResourceEvaluator> meAsList = new ArrayList<>(me);
-				List<? extends RangerPolicyResourceEvaluator> otherAsList = new ArrayList<>(other);
+				List<? extends RangerResourceEvaluator> meAsList    = new ArrayList<>(me);
+				List<? extends RangerResourceEvaluator> otherAsList = new ArrayList<>(other);
+				List<Long>                              myIds       = new ArrayList<>();
+				List<Long>                              otherIds    = new ArrayList<>();
 
-				List<Long> myIds = new ArrayList<>();
-				List<Long> otherIds = new ArrayList<>();
-				for (RangerPolicyResourceEvaluator evaluator : meAsList) {
-					myIds.add(evaluator.getId());
+				for (RangerResourceEvaluator evaluator : meAsList) {
+					if (evaluator instanceof RangerPolicyResourceEvaluator) {
+						myIds.add(((RangerPolicyResourceEvaluator) evaluator).getPolicyId());
+					} else {
+						myIds.add(evaluator.getId());
+					}
 				}
-				for (RangerPolicyResourceEvaluator evaluator : otherAsList) {
-					otherIds.add(evaluator.getId());
+
+				for (RangerResourceEvaluator evaluator : otherAsList) {
+					if (evaluator instanceof RangerPolicyResourceEvaluator) {
+						otherIds.add(((RangerPolicyResourceEvaluator) evaluator).getPolicyId());
+					} else {
+						otherIds.add(evaluator.getId());
+					}
 				}
 
 				ret = compareLongLists(myIds, otherIds);
 			}
 		}
+
 		return ret;
 	}
 
 	private static boolean compareLongLists(List<Long> me, List<Long> other) {
 		return me.size() == CollectionUtils.intersection(me, other).size();
 	}
-
-
 }
 
