@@ -35,8 +35,6 @@ import org.apache.ranger.plugin.policyengine.RangerAccessResult;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
-import com.google.common.collect.Lists;
-
 public class RangerHiveAuditHandler extends RangerDefaultAuditHandler {
 
 	private static final Logger LOG = LoggerFactory.getLogger(RangerDefaultAuditHandler.class);
@@ -51,14 +49,20 @@ public class RangerHiveAuditHandler extends RangerDefaultAuditHandler {
 
 	public static final String CONF_AUDIT_QUERY_REQUEST_SIZE = "xasecure.audit.solr.limit.query.req.size";
 	public static final int DEFAULT_CONF_AUDIT_QUERY_REQUEST_SIZE = Integer.MAX_VALUE;
-	private int requestQuerySize;
-	Collection<AuthzAuditEvent> auditEvents  = null;
-	boolean                     deniedExists = false;
 
-	Set<String> roleOperationCmds = new HashSet<>(Arrays.asList(HiveOperationType.CREATEROLE.name(), HiveOperationType.DROPROLE.name(),
-			HiveOperationType.SHOW_ROLES.name(), HiveOperationType.SHOW_ROLE_GRANT.name(),
-			HiveOperationType.SHOW_ROLE_PRINCIPALS.name(), HiveOperationType.GRANT_ROLE.name(),
-			HiveOperationType.REVOKE_ROLE.name()));
+	private static final Set<String> ROLE_OPS = new HashSet<>();
+
+	static {
+		for (HiveOperationType e : EnumSet.of(HiveOperationType.CREATEROLE, HiveOperationType.DROPROLE,
+				HiveOperationType.SHOW_ROLES, HiveOperationType.SHOW_ROLE_GRANT, HiveOperationType.SHOW_ROLE_PRINCIPALS,
+				HiveOperationType.GRANT_ROLE, HiveOperationType.REVOKE_ROLE)) {
+			ROLE_OPS.add(e.name());
+		}
+	}
+
+	private final int                         requestQuerySize;
+	private final Collection<AuthzAuditEvent> auditEvents  = new ArrayList<>();
+	private       boolean                     deniedExists = false;
 
 	public RangerHiveAuditHandler() {
 		super();
@@ -67,8 +71,10 @@ public class RangerHiveAuditHandler extends RangerDefaultAuditHandler {
 
 	public RangerHiveAuditHandler(Configuration config) {
 		super(config);
-		requestQuerySize = config.getInt(CONF_AUDIT_QUERY_REQUEST_SIZE, DEFAULT_CONF_AUDIT_QUERY_REQUEST_SIZE);
-		requestQuerySize = (requestQuerySize < 1) ? DEFAULT_CONF_AUDIT_QUERY_REQUEST_SIZE : requestQuerySize;
+
+		int configRequestQuerySize = config.getInt(CONF_AUDIT_QUERY_REQUEST_SIZE, DEFAULT_CONF_AUDIT_QUERY_REQUEST_SIZE);
+
+		requestQuerySize = (configRequestQuerySize < 1) ? DEFAULT_CONF_AUDIT_QUERY_REQUEST_SIZE : configRequestQuerySize;
 	}
 
 	AuthzAuditEvent createAuditEvent(RangerAccessResult result, String accessType, String resourcePath) {
@@ -128,7 +134,7 @@ public class RangerHiveAuditHandler extends RangerDefaultAuditHandler {
 			}
 
 			String action = request.getAction();
-			if (hiveResource.getObjectType() == HiveObjectType.GLOBAL && isRoleOperation(action)) {
+			if (hiveResource.getObjectType() == HiveObjectType.GLOBAL && ROLE_OPS.contains(action)) {
 				auditEvent.setAccessType(action);
 			}
 		}
@@ -138,7 +144,7 @@ public class RangerHiveAuditHandler extends RangerDefaultAuditHandler {
 	
 	AuthzAuditEvent createAuditEvent(RangerAccessResult result) {
 
-		AuthzAuditEvent ret = null;
+		final AuthzAuditEvent ret;
 
 		RangerAccessRequest  request  = result.getAccessRequest();
 		RangerAccessResource resource = request.getResource();
@@ -181,6 +187,8 @@ public class RangerHiveAuditHandler extends RangerDefaultAuditHandler {
 			}
 
 			ret = createAuditEvent(result, accessType, resourcePath);
+		} else {
+			ret = null;
 		}
 
 		return ret;
@@ -188,7 +196,7 @@ public class RangerHiveAuditHandler extends RangerDefaultAuditHandler {
 
 	List<AuthzAuditEvent> createAuditEvents(Collection<RangerAccessResult> results) {
 
-		Map<Long, AuthzAuditEvent> auditEvents = new HashMap<Long, AuthzAuditEvent>();
+		Map<Long, AuthzAuditEvent> auditEventsMap = new HashMap<>();
 		Iterator<RangerAccessResult> iterator = results.iterator();
 		AuthzAuditEvent deniedAuditEvent = null;
 		while (iterator.hasNext() && deniedAuditEvent == null) {
@@ -198,8 +206,8 @@ public class RangerHiveAuditHandler extends RangerDefaultAuditHandler {
 					deniedAuditEvent = createAuditEvent(result);
 				} else {
 					long policyId = result.getPolicyId();
-					if (auditEvents.containsKey(policyId)) { // add this result to existing event by updating column values
-						AuthzAuditEvent auditEvent = auditEvents.get(policyId);
+					if (auditEventsMap.containsKey(policyId)) { // add this result to existing event by updating column values
+						AuthzAuditEvent auditEvent = auditEventsMap.get(policyId);
 						RangerHiveAccessRequest request    = (RangerHiveAccessRequest)result.getAccessRequest();
 						RangerHiveResource resource   = (RangerHiveResource)request.getResource();
 						String resourcePath = auditEvent.getResourcePath() + "," + resource.getColumn();
@@ -212,19 +220,15 @@ public class RangerHiveAuditHandler extends RangerDefaultAuditHandler {
 						AuthzAuditEvent auditEvent = createAuditEvent(result);
 
 						if(auditEvent != null) {
-							auditEvents.put(policyId, auditEvent);
+							auditEventsMap.put(policyId, auditEvent);
 						}
 					}
 				}
 			}
 		}
-		List<AuthzAuditEvent> result;
-		if (deniedAuditEvent == null) {
-			result = new ArrayList<>(auditEvents.values());
-		} else {
-			result = Lists.newArrayList(deniedAuditEvent);
-		}
-		
+		final List<AuthzAuditEvent> result = (deniedAuditEvent == null) ? new ArrayList<>(auditEventsMap.values())
+				: Collections.singletonList(deniedAuditEvent);
+
 		return result;
 	}
 
@@ -251,8 +255,8 @@ public class RangerHiveAuditHandler extends RangerDefaultAuditHandler {
 	 */
 	@Override
 	public void processResults(Collection<RangerAccessResult> results) {
-		List<AuthzAuditEvent> auditEvents = createAuditEvents(results);
-		for(AuthzAuditEvent auditEvent : auditEvents) {
+		List<AuthzAuditEvent> result = createAuditEvents(results);
+		for (AuthzAuditEvent auditEvent : result) {
 			addAuthzAuditEvent(auditEvent);
 		}
 	}
@@ -280,33 +284,25 @@ public class RangerHiveAuditHandler extends RangerDefaultAuditHandler {
 		addAuthzAuditEvent(auditEvent);
     }
 
-    public void flushAudit() {
-    	if(auditEvents == null) {
-    		return;
-    	}
+	public void flushAudit() {
+		for (AuthzAuditEvent auditEvent : auditEvents) {
+			if (deniedExists && auditEvent.getAccessResult() != 0) { // if deny exists, skip logging for allowed results
+				continue;
+			}
 
-    	for(AuthzAuditEvent auditEvent : auditEvents) {
-    		if(deniedExists && auditEvent.getAccessResult() != 0) { // if deny exists, skip logging for allowed results
-    			continue;
-    		}
+			super.logAuthzAudit(auditEvent);
+		}
+	}
 
-    		super.logAuthzAudit(auditEvent);
-    	}
-    }
+	private void addAuthzAuditEvent(AuthzAuditEvent auditEvent) {
+		if (auditEvent != null) {
+			auditEvents.add(auditEvent);
 
-    private void addAuthzAuditEvent(AuthzAuditEvent auditEvent) {
-    	if(auditEvent != null) {
-    		if(auditEvents == null) {
-    			auditEvents = new ArrayList<AuthzAuditEvent>();
-    		}
-    		
-    		auditEvents.add(auditEvent);
-    		
-    		if(auditEvent.getAccessResult() == 0) {
-    			deniedExists = true;
-    		}
-    	}
-    }
+			if (auditEvent.getAccessResult() == 0) {
+				deniedExists = true;
+			}
+		}
+	}
 
     private String getReplCmd(String cmdString) {
 		String ret = "REPL";
@@ -353,24 +349,18 @@ public class RangerHiveAuditHandler extends RangerDefaultAuditHandler {
 		return ret;
 	}
 
-	private boolean isRoleOperation(String action) {
-		boolean ret = false;
-		if (roleOperationCmds.contains(action)) {
-			ret = true;
-		}
-		return ret;
-	}
-
 	private String getURLPathString(RangerAccessResource resource, String resourcePath) {
 		String ret = resourcePath;
-		ArrayList<String> resourcePathVal;
 		Object val = resource.getValue(URL_RESOURCE_TYPE);
+
 		if (val instanceof List<?>) {
-			resourcePathVal = (ArrayList<String>) val;
+			List<String> resourcePathVal = (List<String>) val;
+
 			if (CollectionUtils.isNotEmpty(resourcePathVal)) {
-				ret = resourcePathVal.get(0);
+				ret = resourcePathVal.iterator().next();
 			}
 		}
+
 		return ret;
 	}
 }
