@@ -99,18 +99,23 @@ public class KafkaRangerAuthorizerTest {
     	truststorePath = truststoreFile.getPath();
     			
         zkServer = new TestingServer();
-        
+		zkServer.start() ;
+
         // Get a random port
-        ServerSocket serverSocket = new ServerSocket(0);
-        port = serverSocket.getLocalPort();
-        serverSocket.close();
+		try (ServerSocket serverSocket = new ServerSocket(0)) {
+			Assert.assertNotNull(serverSocket) ;
+			port = serverSocket.getLocalPort() ;
+			Assert.assertTrue(port > 0) ;
+		} catch (java.io.IOException e) {
+			throw new RuntimeException("Local socket port not available", e) ;
+		}
 
         tempDir = Files.createTempDirectory("kafka");
         
         final Properties props = new Properties();
-        props.put("broker.id", 1);
+        props.put("broker.id", String.valueOf(1)) ;
         props.put("host.name", "localhost");
-        props.put("port", port);
+        props.put("port", String.valueOf(port));
         props.put("log.dir", tempDir.toString());
         props.put("zookeeper.connect", zkServer.getConnectString());
         props.put("replica.socket.timeout.ms", "1500");
