@@ -475,7 +475,7 @@ public class RangerSolrAuthorizer extends SearchComponent implements Authorizati
 		if (httpServletRequest == null) {
 			SolrCore solrCore = rb.req.getCore();
 			StringBuilder builder = new StringBuilder("Unable to locate HttpServletRequest");
-			if (solrCore != null && !solrCore.getSolrConfig().getBool("requestDispatcher/requestParsers/@addHttpRequestToContext", true)) {
+			if (solrCore != null && !solrCore.getSolrConfig().get("requestDispatcher/requestParsers/@addHttpRequestToContext").boolVal(true)) {
 				builder.append(", ensure requestDispatcher/requestParsers/@addHttpRequestToContext is set to true in solrconfig.xml");
 			}
 			throw new SolrException(SolrException.ErrorCode.UNAUTHORIZED, builder.toString());
@@ -758,7 +758,7 @@ public class RangerSolrAuthorizer extends SearchComponent implements Authorizati
 		HttpServletRequest httpServletRequest = (HttpServletRequest) req.getContext().get("httpRequest");
 		if (httpServletRequest == null) {
 			StringBuilder builder = new StringBuilder("Unable to locate HttpServletRequest");
-			if (solrCore != null && !solrCore.getSolrConfig().getBool("requestDispatcher/requestParsers/@addHttpRequestToContext", true)) {
+			if (solrCore != null && !solrCore.getSolrConfig().get("requestDispatcher/requestParsers/@addHttpRequestToContext").boolVal(true)) {
 				builder.append(", ensure requestDispatcher/requestParsers/@addHttpRequestToContext is set to true in solrconfig.xml");
 			}
 			throw new SolrException(SolrException.ErrorCode.UNAUTHORIZED, builder.toString());
