@@ -23,8 +23,6 @@ import java.util.Date;
 import java.util.List;
 
 import org.apache.commons.collections.CollectionUtils;
-import org.apache.commons.logging.Log;
-import org.apache.commons.logging.LogFactory;
 import org.apache.ranger.plugin.audit.RangerDefaultAuditHandler;
 import org.apache.ranger.plugin.policyengine.RangerAccessRequestImpl;
 import org.apache.ranger.plugin.policyengine.RangerAccessResourceImpl;
@@ -37,11 +35,13 @@ import org.apache.sqoop.model.MPrivilege;
 import org.apache.sqoop.model.MResource;
 import org.apache.sqoop.security.AuthorizationValidator;
 import org.apache.sqoop.security.SecurityError;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
 import com.google.common.collect.Sets;
 
 public class RangerSqoopAuthorizer extends AuthorizationValidator {
-	private static final Log LOG = LogFactory.getLog(RangerSqoopAuthorizer.class);
+	private static final Logger LOG = LoggerFactory.getLogger(RangerSqoopAuthorizer.class);
 
 	private static volatile RangerSqoopPlugin sqoopPlugin = null;
 
@@ -145,7 +145,7 @@ class RangerSqoopPlugin extends RangerBasePlugin {
 	public void init() {
 		super.init();
 
-		RangerDefaultAuditHandler auditHandler = new RangerDefaultAuditHandler();
+		RangerDefaultAuditHandler auditHandler = new RangerDefaultAuditHandler(getConfig());
 
 		super.setResultProcessor(auditHandler);
 	}
@@ -182,4 +182,4 @@ class RangerSqoopAccessRequest extends RangerAccessRequestImpl {
 		super.setAccessTime(new Date());
 		super.setClientIPAddress(clientIPAddress);
 	}
-}
\ No newline at end of file
+}
