@@ -6,9 +6,9 @@
  * to you under the Apache License, Version 2.0 (the
  * "License"); you may not use this file except in compliance
  * with the License.  You may obtain a copy of the License at
- * 
+ *
  * http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing,
  * software distributed under the License is distributed on an
  * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
@@ -21,13 +21,8 @@ package org.apache.ranger.services.yarn.client;
 
 import java.util.Map;
 
-import org.apache.log4j.Logger;
-
-
 public class YarnConnectionMgr {
 
-	public static final Logger LOG = Logger.getLogger(YarnConnectionMgr.class);
-	
 	public static YarnClient getYarnClient(String serviceName, Map<String, String> configs) {
         return  new YarnClient(serviceName, configs);
     }
