@@ -33,7 +33,6 @@ import javax.servlet.http.HttpSession;
 import org.apache.commons.collections.CollectionUtils;
 import org.apache.commons.lang.StringUtils;
 import org.apache.commons.lang.time.DateUtils;
-import org.apache.log4j.Logger;
 import org.apache.ranger.common.DateUtil;
 import org.apache.ranger.common.HTTPUtil;
 import org.apache.ranger.common.MessageEnums;
@@ -65,12 +64,14 @@ import org.springframework.security.web.authentication.WebAuthenticationDetails;
 import org.springframework.stereotype.Component;
 import org.springframework.transaction.annotation.Propagation;
 import org.springframework.transaction.annotation.Transactional;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
 @Component
 @Transactional
 public class SessionMgr {
 
-	static final Logger logger = Logger.getLogger(SessionMgr.class);
+	static final Logger logger = LoggerFactory.getLogger(SessionMgr.class);
 
 	@Autowired
 	RESTErrorUtil restErrorUtil;
@@ -176,21 +177,36 @@ public class SessionMgr {
 				userSession.setSpnegoEnabled(true);
 			}
 
+			Boolean ssoEnabled;
+			if (authType == XXAuthSession.AUTH_TYPE_TRUSTED_PROXY) {
+				ssoEnabled = true;
+			} else {
+				Object ssoEnabledObj = httpRequest.getAttribute("ssoEnabled");
+				ssoEnabled = ssoEnabledObj != null ? Boolean.valueOf(String.valueOf(ssoEnabledObj)) : PropertiesUtil.getBooleanProperty("ranger.sso.enabled", false);
+			}
+
+			if (logger.isDebugEnabled()) {
+				logger.debug("session id = " + userSession.getLoginId() + " ssoenabled = " + ssoEnabled);
+			}
+			userSession.setSSOEnabled(ssoEnabled);
+
 			resetUserSessionForProfiles(userSession);
 			resetUserModulePermission(userSession);
 
 			Calendar cal = Calendar.getInstance();
-			if (details != null) {
-				logger.info("Login Success: loginId=" + currentLoginId
-						+ ", sessionId=" + gjAuthSession.getId()
-						+ ", sessionId=" + details.getSessionId()
-						+ ", requestId=" + details.getRemoteAddress()
-						+ ", epoch=" + cal.getTimeInMillis());
-			} else {
-				logger.info("Login Success: loginId=" + currentLoginId
-						+ ", sessionId=" + gjAuthSession.getId()
-						+ ", details is null"
-						+ ", epoch=" + cal.getTimeInMillis());
+			if(logger.isDebugEnabled()) {
+				if (details != null) {
+					logger.debug("Login Success: loginId=" + currentLoginId
+							+ ", sessionId=" + gjAuthSession.getId()
+							+ ", sessionId=" + details.getSessionId()
+							+ ", requestId=" + details.getRemoteAddress()
+							+ ", epoch=" + cal.getTimeInMillis());
+				} else {
+					logger.debug("Login Success: loginId=" + currentLoginId
+							+ ", sessionId=" + gjAuthSession.getId()
+							+ ", details is null"
+							+ ", epoch=" + cal.getTimeInMillis());
+				}
 			}
 
 		}
@@ -229,7 +245,9 @@ public class SessionMgr {
 			rangerUserPermission.setUserPermissions(userPermissions);
 			rangerUserPermission.setLastUpdatedTime(Calendar.getInstance().getTimeInMillis());
 			userSession.setRangerUserPermission(rangerUserPermission);
-			logger.info("UserSession Updated to set new Permissions to User: " + userSession.getLoginId());
+			if (logger.isDebugEnabled()) {
+				logger.debug("UserSession Updated to set new Permissions to User: " + userSession.getLoginId());
+			}
 		} else {
 			logger.error("No XUser found with username: " + userSession.getLoginId() + "So Permission is not set for the user");
 		}
@@ -314,7 +332,7 @@ public class SessionMgr {
 				.equalsIgnoreCase(userSession.getXXPortalUser().getLoginId())) {
 			return true;
 		} else {
-			logger.info(
+			logger.warn(
 					"loginId doesn't match loginId from HTTPSession. Will create new session. loginId="
 							+ currentLoginId + ", userSession=" + userSession,
 					new Exception());
@@ -435,14 +453,42 @@ public class SessionMgr {
 		VXAuthSession vXAuthSession = authSessionService.populateViewBean(xXAuthSession);
 		return vXAuthSession;
 	}
-	
+
+	/**
+	 * Check whether the user failed to log in so many times that we need to lock it for
+	 * a while. The current limit of is to fail at most n times in a sliding time window,
+	 * otherwise the login verification will not be performed in the future.
+	 * @param loginId
+	 * @return
+	 */
+	public boolean isLoginIdLocked(String loginId) {
+		boolean ret             = false;
+		boolean autoLockEnabled = PropertiesUtil.getBooleanProperty("ranger.admin.login.autolock.enabled", true);
+
+		if (autoLockEnabled) {
+			int  windowSeconds    = PropertiesUtil.getIntProperty("ranger.admin.login.autolock.window.seconds", 300);
+			int  maxFailuresCount = PropertiesUtil.getIntProperty("ranger.admin.login.autolock.maxfailure", 5);
+			long failuresCount    = daoManager.getXXAuthSession().getRecentAuthFailureCountByLoginId(loginId, windowSeconds);
+
+			ret = failuresCount >= maxFailuresCount;
+
+			if (logger.isDebugEnabled()) {
+				logger.debug("isLoginIdLocked(loginId={}): windowSeconds={}, maxFailuresCount={}, failuresCount={}, ret={}", loginId, windowSeconds, maxFailuresCount, failuresCount, ret);
+			}
+		}
+
+		return ret;
+	}
+
 	public boolean isValidXAUser(String loginId) {
 		XXPortalUser pUser = daoManager.getXXPortalUser().findByLoginId(loginId);
 		if (pUser == null) {
 			logger.error("Error getting user for loginId=" + loginId);
 			return false;
 		} else {
-			logger.info(loginId+" is a valid user");
+			if(logger.isDebugEnabled()) {
+				logger.debug(loginId + " is a valid user");
+			}
 			return true;
 		}
 		
