@@ -37,9 +37,11 @@ public class RangerUserStoreCache {
 	private static final int MAX_WAIT_TIME_FOR_UPDATE = 10;
 
 	public static volatile RangerUserStoreCache 	sInstance = null;
-	private final int 								waitTimeInSeconds;
-	private final ReentrantLock 					lock = new ReentrantLock();
-	private RangerUserStore 						rangerUserStore;
+
+	private final int             waitTimeInSeconds;
+	private final boolean         dedupStrings;
+	private final ReentrantLock   lock = new ReentrantLock();
+	private       RangerUserStore rangerUserStore;
 
 	public static RangerUserStoreCache getInstance() {
 		if (sInstance == null) {
@@ -54,66 +56,72 @@ public class RangerUserStoreCache {
 
 	private RangerUserStoreCache() {
 		RangerAdminConfig config = RangerAdminConfig.getInstance();
-		waitTimeInSeconds = config.getInt("ranger.admin.userstore.download.cache.max.waittime.for.update", MAX_WAIT_TIME_FOR_UPDATE);
-		this.rangerUserStore = new RangerUserStore();
+
+		this.waitTimeInSeconds = config.getInt("ranger.admin.userstore.download.cache.max.waittime.for.update", MAX_WAIT_TIME_FOR_UPDATE);
+		this.dedupStrings      = config.getBoolean("ranger.admin.userstore.dedup.strings", Boolean.TRUE);
+		this.rangerUserStore   = new RangerUserStore();
 	}
 
 	public RangerUserStore getRangerUserStore() {
 		return this.rangerUserStore;
 	}
 
-	public RangerUserStore getLatestRangerUserStoreOrCached(XUserMgr xUserMgr, Long lastKnownUserStoreVersion, Long rangerUserStoreVersionInDB) throws Exception {
-		RangerUserStore ret = null;
-
-		if (lastKnownUserStoreVersion == null || !lastKnownUserStoreVersion.equals(rangerUserStoreVersionInDB)) {
-			ret = getLatestRangerUserStore(xUserMgr, lastKnownUserStoreVersion, rangerUserStoreVersionInDB);
-		} else if (lastKnownUserStoreVersion.equals(rangerUserStoreVersionInDB)) {
-			ret = null;
-		} else {
-			ret = getRangerUserStore();
-		}
-
-		return ret;
-	}
-
-	public RangerUserStore getLatestRangerUserStore(XUserMgr xUserMgr, Long lastKnownUserStoreVersion, Long rangerUserStoreVersionInDB) throws Exception {
-		RangerUserStore ret	 = null;
-		boolean         lockResult   = false;
+	public RangerUserStore getLatestRangerUserStoreOrCached(XUserMgr xUserMgr) throws Exception {
 		if (LOG.isDebugEnabled()) {
-			LOG.debug("==> RangerUserStoreCache.getLatestRangerUserStore(lastKnownUserStoreVersion= " + lastKnownUserStoreVersion + " rangerUserStoreVersionInDB= " + rangerUserStoreVersionInDB + ")");
+			LOG.debug("==> RangerUserStoreCache.getLatestRangerUserStoreOrCached()");
 		}
 
+		RangerUserStore ret        = null;
+		boolean         lockResult = false;
+
 		try {
 			lockResult = lock.tryLock(waitTimeInSeconds, TimeUnit.SECONDS);
 
 			if (lockResult) {
-				final Set<UserInfo> rangerUsersInDB = xUserMgr.getUsers();
-				final Set<GroupInfo> rangerGroupsInDB = xUserMgr.getGroups();
-				final Map<String, Set<String>> userGroups = xUserMgr.getUserGroups();
-				if (LOG.isDebugEnabled()) {
-					LOG.debug("No. of users from DB = " + rangerUsersInDB.size() + " and no. of groups from DB = " + rangerGroupsInDB.size());
-					LOG.debug("No. of userGroupMappings = " + userGroups.size());
-				}
+				Long cachedUserStoreVersion = rangerUserStore.getUserStoreVersion();
+				Long dbUserStoreVersion     = xUserMgr.getUserStoreVersion();
 
-				ret = new RangerUserStore(rangerUserStoreVersionInDB, rangerUsersInDB, rangerGroupsInDB, userGroups);
-				rangerUserStore = ret;
+				if (!Objects.equals(cachedUserStoreVersion, dbUserStoreVersion)) {
+					LOG.info("RangerUserStoreCache refreshing from version " + cachedUserStoreVersion + " to " + dbUserStoreVersion);
+
+					final Set<UserInfo>            rangerUsersInDB  = xUserMgr.getUsers();
+					final Set<GroupInfo>           rangerGroupsInDB = xUserMgr.getGroups();
+					final Map<String, Set<String>> userGroups       = xUserMgr.getUserGroups();
+
+					if (LOG.isDebugEnabled()) {
+						LOG.debug("No. of users from DB = " + rangerUsersInDB.size() + " and no. of groups from DB = " + rangerGroupsInDB.size());
+						LOG.debug("No. of userGroupMappings = " + userGroups.size());
+					}
+
+					RangerUserStore rangerUserStore = new RangerUserStore(dbUserStoreVersion, rangerUsersInDB, rangerGroupsInDB, userGroups);
+
+					if (dedupStrings) {
+						rangerUserStore.dedupStrings();
+					}
+
+					this.rangerUserStore = rangerUserStore;
+
+					LOG.info("RangerUserStoreCache refreshed from version " + cachedUserStoreVersion + " to " + dbUserStoreVersion + ": users=" + rangerUsersInDB.size() + ", groups=" + rangerGroupsInDB.size() + ", userGroupMappings=" + userGroups.size());
+				}
 			} else {
 				if (LOG.isDebugEnabled()) {
 					LOG.debug("Could not get lock in [" + waitTimeInSeconds + "] seconds, returning cached RangerUserStore");
 				}
-				ret = getRangerUserStore();
 			}
 		} catch (InterruptedException exception) {
-			LOG.error("RangerUserStoreCache.getLatestRangerUserStore:lock got interrupted..", exception);
+			LOG.error("RangerUserStoreCache.getLatestRangerUserStoreOrCached:lock got interrupted..", exception);
 		} finally {
+			ret = rangerUserStore;
+
 			if (lockResult) {
 				lock.unlock();
 			}
 		}
 
 		if (LOG.isDebugEnabled()) {
-			LOG.debug("<== RangerUserStoreCache.getLatestRangerUserStore(lastKnownUserStoreVersion= " + lastKnownUserStoreVersion + " rangerUserStoreVersionInDB= " + rangerUserStoreVersionInDB + " RangerUserStore= " + ret + ")");
+			LOG.debug("<== RangerUserStoreCache.getLatestRangerUserStoreOrCached(): ret=" + ret);
 		}
+
 		return ret;
 	}
 }
