@@ -18,7 +18,6 @@
 package org.apache.ranger.patch;
 
 import org.apache.commons.lang.StringUtils;
-import org.apache.log4j.Logger;
 import org.apache.ranger.authorization.utils.JsonUtils;
 import org.apache.ranger.biz.RangerBizUtil;
 import org.apache.ranger.biz.ServiceDBStore;
@@ -49,6 +48,8 @@ import org.apache.ranger.service.RangerPolicyService;
 import org.apache.ranger.service.XPermMapService;
 import org.apache.ranger.service.XPolicyService;
 import org.apache.ranger.util.CLIUtil;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 import org.springframework.beans.factory.annotation.Autowired;
 import org.springframework.stereotype.Component;
 
@@ -61,7 +62,7 @@ import java.util.Map;
 
 @Component
 public class PatchForKafkaServiceDefUpdate_J10025 extends BaseLoader {
-	private static final Logger logger = Logger.getLogger(PatchForKafkaServiceDefUpdate_J10025.class);
+	private static final Logger logger = LoggerFactory.getLogger(PatchForKafkaServiceDefUpdate_J10025.class);
 	private static final List<String> POLICY_NAMES = new ArrayList<>(Arrays.asList("all - cluster", "all - delegationtoken"));
 	private static final String LOGIN_ID_ADMIN = "admin";
 	private static final String KAFKA_RESOURCE_CLUSTER = "cluster";
@@ -446,4 +447,4 @@ public class PatchForKafkaServiceDefUpdate_J10025 extends BaseLoader {
 		}
 		return rangerPolicyItemAccesses;
 	}
-}
\ No newline at end of file
+}
