@@ -28,7 +28,6 @@ import java.util.Set;
 
 import org.apache.commons.collections.CollectionUtils;
 import org.apache.commons.lang.StringUtils;
-import org.apache.log4j.Logger;
 import org.apache.ranger.biz.RangerBizUtil;
 import org.apache.ranger.biz.ServiceDBStore;
 import org.apache.ranger.common.AppConstants;
@@ -59,12 +58,14 @@ import org.apache.ranger.service.XPermMapService;
 import org.apache.ranger.service.XPolicyService;
 import org.apache.ranger.util.CLIUtil;
 import org.apache.ranger.view.VXPermMap;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 import org.springframework.beans.factory.annotation.Autowired;
 import org.springframework.stereotype.Component;
 
 @Component
 public class PatchMigration_J10002 extends BaseLoader {
-	private static final Logger logger = Logger.getLogger(PatchMigration_J10002.class);
+	private static final Logger logger = LoggerFactory.getLogger(PatchMigration_J10002.class);
 
 	@Autowired
 	RangerDaoManager daoMgr;
@@ -148,10 +149,10 @@ public class PatchMigration_J10002 extends BaseLoader {
 		try {
 			List<XXAsset> repoList = daoMgr.getXXAsset().getAll();
 
-			if (repoList.size() <= 0) {
+			if (repoList.isEmpty()) {
 				return;
 			}
-			if (repoList.size() > 0) {
+			if (!repoList.isEmpty()) {
 				EmbeddedServiceDefsUtil.instance().init(svcDBStore);
 			}
 
@@ -188,7 +189,7 @@ public class PatchMigration_J10002 extends BaseLoader {
 
 		try {
 			List<XXResource> resList = daoMgr.getXXResource().getAll();
-			if (resList.size() <= 0) {
+			if (resList.isEmpty()) {
 				return;
 			}
 
@@ -405,9 +406,7 @@ public class PatchMigration_J10002 extends BaseLoader {
 				ret.put(resourceType, resource);
 			}
 
-			for(String res : resourceString.split(",")) {
-				resource.getValues().add(res);
-			}
+			Collections.addAll(resource.getValues(), resourceString.split(","));
 		}
 
 		return ret;
