@@ -18,21 +18,23 @@
 
 package org.apache.ranger.patch.cliutil;
 
-import org.apache.log4j.Logger;
 import org.apache.ranger.biz.UserMgr;
 import org.apache.ranger.common.MessageEnums;
 import org.apache.ranger.common.RESTErrorUtil;
+import org.apache.ranger.common.StringUtil;
 import org.apache.ranger.db.RangerDaoManager;
 import org.apache.ranger.entity.XXPortalUser;
 import org.apache.ranger.patch.BaseLoader;
 import org.apache.ranger.util.CLIUtil;
-import org.apache.solr.common.StringUtils;
+import org.apache.commons.lang.StringUtils;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 import org.springframework.beans.factory.annotation.Autowired;
 import org.springframework.stereotype.Component;
 
 @Component
 public class ChangePasswordUtil extends BaseLoader {
-	private static final Logger logger = Logger.getLogger(ChangePasswordUtil.class);
+	private static final Logger logger = LoggerFactory.getLogger(ChangePasswordUtil.class);
 
 	@Autowired
 	RangerDaoManager daoMgr;
@@ -132,30 +134,50 @@ public class ChangePasswordUtil extends BaseLoader {
 			String currentEncryptedPassword = null;
 			String md5EncryptedPassword = null;
 			try {
-				currentEncryptedPassword = userMgr.encrypt(userLoginId, currentPassword);
-				if (currentEncryptedPassword.equals(dbPassword)) {
-					validatePassword(newPassword);
-					userMgr.updatePasswordInSHA256(userLoginId, newPassword, true);
-					logger.info("User '" + userLoginId + "' Password updated sucessfully.");
-				}
-				else if (!currentEncryptedPassword.equals(dbPassword) && defaultPwdChangeRequest) {
-					logger.info("current encryped password is not equal to dbpassword , trying with md5 now");
-					md5EncryptedPassword = userMgr.encryptWithOlderAlgo(userLoginId, currentPassword);
-					if (md5EncryptedPassword.equals(dbPassword)) {
+				if (config.isFipsEnabled()) {
+					if (defaultPwdChangeRequest) {
+						md5EncryptedPassword = userMgr.encryptWithOlderAlgo(userLoginId, currentPassword);
+						if (md5EncryptedPassword.equals(dbPassword)) {
+							validatePassword(newPassword);
+							userMgr.updatePasswordInSHA256(userLoginId, newPassword, true);
+							logger.info("User '" + userLoginId + "' Password updated sucessfully.");
+						} else {
+							System.out.println(
+									"Skipping default password change request as provided password doesn't match with existing password.");
+							logger.error(
+									"Skipping default password change request as provided password doesn't match with existing password.");
+							System.exit(2);
+						}
+					} else if (userMgr.isPasswordValid(userLoginId, dbPassword, currentPassword)) {
 						validatePassword(newPassword);
 						userMgr.updatePasswordInSHA256(userLoginId, newPassword, true);
 						logger.info("User '" + userLoginId + "' Password updated sucessfully.");
-					} else {
-						System.out.println(
-								"Skipping default password change request as provided password doesn't match with existing password.");
-						logger.error(
-								"Skipping default password change request as provided password doesn't match with existing password.");
-						System.exit(2);
 					}
 				} else {
-					System.out.println("Invalid user password");
-					logger.error("Invalid user password");
-					System.exit(1);
+					currentEncryptedPassword = userMgr.encrypt(userLoginId, currentPassword);
+					if (currentEncryptedPassword.equals(dbPassword)) {
+						validatePassword(newPassword);
+						userMgr.updatePasswordInSHA256(userLoginId, newPassword, true);
+						logger.info("User '" + userLoginId + "' Password updated sucessfully.");
+					} else if (!currentEncryptedPassword.equals(dbPassword) && defaultPwdChangeRequest) {
+						logger.info("current encryped password is not equal to dbpassword , trying with md5 now");
+						md5EncryptedPassword = userMgr.encryptWithOlderAlgo(userLoginId, currentPassword);
+						if (md5EncryptedPassword.equals(dbPassword)) {
+							validatePassword(newPassword);
+							userMgr.updatePasswordInSHA256(userLoginId, newPassword, true);
+							logger.info("User '" + userLoginId + "' Password updated sucessfully.");
+						} else {
+							System.out.println(
+									"Skipping default password change request as provided password doesn't match with existing password.");
+							logger.error(
+									"Skipping default password change request as provided password doesn't match with existing password.");
+							System.exit(2);
+						}
+					} else {
+						System.out.println("Invalid user password");
+						logger.error("Invalid user password");
+						System.exit(1);
+					}
 				}
 			} catch (Exception e) {
 				logger.error("Update Admin Password failure. Detail:  \n", e);
@@ -239,16 +261,13 @@ public class ChangePasswordUtil extends BaseLoader {
 	private void validatePassword(String newPassword) {
 		boolean checkPassword = false;
 		if (newPassword != null) {
-			String pattern = "(?=.*[0-9])(?=.*[a-zA-Z]).{8,}";
-			checkPassword = newPassword.trim().matches(pattern);
+			checkPassword = newPassword.trim().matches(StringUtil.VALIDATION_CRED);
 			if (!checkPassword) {
-				logger.error(
-						"validatePassword(). Password should be minimum 8 characters with minimum one alphabet and one numeric.");
-				System.out.println(
-						"validatePassword(). Password should be minimum 8 characters with minimum one alphabet and one numeric.");
+				String msg = "Password should be minimum 8 characters, at least one uppercase letter, one lowercase letter and one numeric.";
+				logger.error(msg);
+				System.out.println(msg);
 				throw restErrorUtil.createRESTException("serverMsg.changePasswordValidatePassword",
-						MessageEnums.INVALID_PASSWORD, null,
-						"Password should be minimum 8 characters with minimum one alphabet and one numeric", null);
+						MessageEnums.INVALID_PASSWORD, null, msg, null);
 			}
 		} else {
 			logger.error("validatePassword(). Password cannot be blank/null.");
