@@ -26,6 +26,7 @@ import java.util.List;
 
 import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpServletResponse;
+import javax.ws.rs.Consumes;
 import javax.ws.rs.DELETE;
 import javax.ws.rs.Encoded;
 import javax.ws.rs.GET;
@@ -129,7 +130,7 @@ public class AssetREST {
 	
 	@GET
 	@Path("/assets/{id}")
-	@Produces({ "application/json", "application/xml" })
+	@Produces({ "application/json" })
 	@PreAuthorize("@rangerPreAuthSecurityHandler.isAPIAccessible(\"" + RangerAPIList.GET_X_ASSET + "\")")
 	public VXAsset getXAsset(@PathParam("id") Long id) {
 		if(logger.isDebugEnabled()) {
@@ -149,7 +150,8 @@ public class AssetREST {
 
 	@POST
 	@Path("/assets")
-	@Produces({ "application/json", "application/xml" })
+	@Consumes({ "application/json" })
+	@Produces({ "application/json" })
 	@PreAuthorize("@rangerPreAuthSecurityHandler.isAPIAccessible(\"" + RangerAPIList.CREATE_X_ASSET + "\")")
 	public VXAsset createXAsset(VXAsset vXAsset) {
 		if(logger.isDebugEnabled()) {
@@ -171,7 +173,8 @@ public class AssetREST {
 
 	@PUT
 	@Path("/assets/{id}")
-	@Produces({ "application/json", "application/xml" })
+	@Consumes({ "application/json" })
+	@Produces({ "application/json" })
 	@PreAuthorize("@rangerPreAuthSecurityHandler.isAPIAccessible(\"" + RangerAPIList.UPDATE_X_ASSET + "\")")
 	public VXAsset updateXAsset(VXAsset vXAsset) {
 		if(logger.isDebugEnabled()) {
@@ -210,7 +213,8 @@ public class AssetREST {
 
 	@POST
 	@Path("/assets/testConfig")
-	@Produces({ "application/json", "application/xml" })
+	@Consumes({ "application/json" })
+	@Produces({ "application/json" })
 	@PreAuthorize("@rangerPreAuthSecurityHandler.isAPIAccessible(\"" + RangerAPIList.TEST_CONFIG + "\")")
 	public VXResponse configTest(VXAsset vXAsset) {
 		if(logger.isDebugEnabled()) {
@@ -230,7 +234,7 @@ public class AssetREST {
 
 	@GET
 	@Path("/assets")
-	@Produces({ "application/json", "application/xml" })
+	@Produces({ "application/json" })
 	@PreAuthorize("@rangerPreAuthSecurityHandler.isAPIAccessible(\"" + RangerAPIList.SEARCH_X_ASSETS + "\")")
 	public VXAssetList searchXAssets(@Context HttpServletRequest request) {
 		if(logger.isDebugEnabled()) {
@@ -266,7 +270,7 @@ public class AssetREST {
 
 	@GET
 	@Path("/assets/count")
-	@Produces({ "application/json", "application/xml" })
+	@Produces({ "application/json" })
 	@PreAuthorize("@rangerPreAuthSecurityHandler.isAPIAccessible(\"" + RangerAPIList.COUNT_X_ASSETS + "\")")
 	public VXLong countXAssets(@Context HttpServletRequest request) {
 		if(logger.isDebugEnabled()) {
@@ -286,7 +290,7 @@ public class AssetREST {
 
 	@GET
 	@Path("/resources/{id}")
-	@Produces({ "application/json", "application/xml" })
+	@Produces({ "application/json" })
 	public VXResource getXResource(@PathParam("id") Long id) {
 		if(logger.isDebugEnabled()) {
 			logger.debug("==> AssetREST.getXResource(" + id + ")");
@@ -312,7 +316,8 @@ public class AssetREST {
 
 	@POST
 	@Path("/resources")
-	@Produces({ "application/json", "application/xml" })
+	@Consumes({ "application/json" })
+	@Produces({ "application/json" })
 	public VXResource createXResource(VXResource vXResource) {
 		if(logger.isDebugEnabled()) {
 			logger.debug("==> AssetREST.createXResource(" + vXResource + ")");
@@ -334,16 +339,24 @@ public class AssetREST {
 	
 	@PUT
 	@Path("/resources/{id}")
-	@Produces({ "application/json", "application/xml" })
-	public VXResource updateXResource(VXResource vXResource) {
+	@Consumes({ "application/json" })
+	@Produces({ "application/json" })
+	public VXResource updateXResource(VXResource vXResource , @PathParam("id") Long id) {
 		if(logger.isDebugEnabled()) {
 			logger.debug("==> AssetREST.updateXResource(" + vXResource + ")");
 		}
 
+		// if vXResource.id is specified, it should be same as the param 'id'
+		if (vXResource.getId() == null) {
+			vXResource.setId(id);
+		} else if(!vXResource.getId().equals(id)) {
+			throw restErrorUtil.createRESTException(HttpServletResponse.SC_BAD_REQUEST , "resource Id mismatch", true);
+		}
+
 		RangerService service = serviceREST.getService(vXResource.getAssetId());
 		RangerPolicy  policy  = serviceUtil.toRangerPolicy(vXResource, service);
 
-		RangerPolicy updatedPolicy = serviceREST.updatePolicy(policy);
+		RangerPolicy updatedPolicy = serviceREST.updatePolicy(policy, policy.getId());
 		
 		VXResource ret = serviceUtil.toVXResource(updatedPolicy, service);
 
@@ -373,7 +386,7 @@ public class AssetREST {
 
 	@GET
 	@Path("/resources")
-	@Produces({ "application/json", "application/xml" })
+	@Produces({ "application/json" })
 	public VXResourceList searchXResources(@Context HttpServletRequest request) {
 		if(logger.isDebugEnabled()) {
 			logger.debug("==> AssetREST.searchXResources()");
@@ -410,7 +423,7 @@ public class AssetREST {
 
 	@GET
 	@Path("/resources/count")
-	@Produces({ "application/json", "application/xml" })
+	@Produces({ "application/json" })
 	public VXLong countXResources(@Context HttpServletRequest request) {
 		if(logger.isDebugEnabled()) {
 			logger.debug("==> AssetREST.countXResources()");
@@ -429,14 +442,15 @@ public class AssetREST {
 
 	@GET
 	@Path("/credstores/{id}")
-	@Produces({ "application/json", "application/xml" })
+	@Produces({ "application/json" })
 	public VXCredentialStore getXCredentialStore(@PathParam("id") Long id) {
 		return assetMgr.getXCredentialStore(id);
 	}
 
 	@POST
 	@Path("/credstores")
-	@Produces({ "application/json", "application/xml" })
+	@Consumes({ "application/json" })
+	@Produces({ "application/json" })
 	public VXCredentialStore createXCredentialStore(
 			VXCredentialStore vXCredentialStore) {
 		return assetMgr.createXCredentialStore(vXCredentialStore);
@@ -444,7 +458,8 @@ public class AssetREST {
 
 	@PUT
 	@Path("/credstores")
-	@Produces({ "application/json", "application/xml" })
+	@Consumes({ "application/json" })
+	@Produces({ "application/json" })
 	public VXCredentialStore updateXCredentialStore(
 			VXCredentialStore vXCredentialStore) {
 		return assetMgr.updateXCredentialStore(vXCredentialStore);
@@ -462,7 +477,7 @@ public class AssetREST {
 
 	@GET
 	@Path("/credstores")
-	@Produces({ "application/json", "application/xml" })
+	@Produces({ "application/json" })
 	public VXCredentialStoreList searchXCredentialStores(
 			@Context HttpServletRequest request) {
 		SearchCriteria searchCriteria = searchUtil.extractCommonCriterias(
@@ -472,7 +487,7 @@ public class AssetREST {
 
 	@GET
 	@Path("/credstores/count")
-	@Produces({ "application/json", "application/xml" })
+	@Produces({ "application/json" })
 	public VXLong countXCredentialStores(@Context HttpServletRequest request) {
 		SearchCriteria searchCriteria = searchUtil.extractCommonCriterias(
 				request, xCredentialStoreService.sortFields);
@@ -481,6 +496,7 @@ public class AssetREST {
 
 	@GET
 	@Path("/resource/{id}")
+	@Produces({ "application/json" })
 	public Response getXResourceFile(@Context HttpServletRequest request,
 			@PathParam("id") Long id) {
 		String fileType = searchUtil.extractString(request,
@@ -504,6 +520,7 @@ public class AssetREST {
 
 	@GET
 	@Path("/policyList/{repository}")
+	@Produces({ "application/json" })
 	@Encoded
 	public String getResourceJSON(@Context HttpServletRequest request,
 			@PathParam("repository") String repository) {
@@ -551,7 +568,7 @@ public class AssetREST {
 
 	@GET
 	@Path("/exportAudit")
-	@Produces({ "application/json", "application/xml" })
+	@Produces({ "application/json" })
 	@PreAuthorize("@rangerPreAuthSecurityHandler.isAPIAccessible(\"" + RangerAPIList.SEARCH_X_POLICY_EXPORT_AUDITS + "\")")
 	public VXPolicyExportAuditList searchXPolicyExportAudits(
 			@Context HttpServletRequest request) {
@@ -582,7 +599,7 @@ public class AssetREST {
 
 	@GET
 	@Path("/report")
-	@Produces({ "application/json", "application/xml" })
+	@Produces({ "application/json" })
 	@PreAuthorize("@rangerPreAuthSecurityHandler.isAPIAccessible(\"" + RangerAPIList.GET_REPORT_LOGS + "\")")
 	public VXTrxLogList getReportLogs(@Context HttpServletRequest request){
 
@@ -604,7 +621,7 @@ public class AssetREST {
 	
 	@GET
 	@Path("/report/{transactionId}")
-	@Produces({ "application/json", "application/xml" })
+	@Produces({ "application/json" })
 	@PreAuthorize("@rangerPreAuthSecurityHandler.isAPIAccessible(\"" + RangerAPIList.GET_TRANSACTION_REPORT + "\")")
 	public VXTrxLogList getTransactionReport(@Context HttpServletRequest request,
 			@PathParam("transactionId") String transactionId){
@@ -613,7 +630,7 @@ public class AssetREST {
 	
 	@GET
 	@Path("/accessAudit")
-	@Produces({ "application/json", "application/xml" })
+	@Produces({ "application/json" })
 	@PreAuthorize("@rangerPreAuthSecurityHandler.isAPIAccessible(\"" + RangerAPIList.GET_ACCESS_LOGS + "\")")
 	public VXAccessAuditList getAccessLogs(@Context HttpServletRequest request){
 		SearchCriteria searchCriteria = searchUtil.extractCommonCriterias(
@@ -674,7 +691,8 @@ public class AssetREST {
 	
 	@POST
 	@Path("/resources/grant")
-	@Produces({ "application/json", "application/xml" })
+	@Consumes({ "application/json" })
+	@Produces({ "application/json" })
 	public VXPolicy grantPermission(@Context HttpServletRequest request,VXPolicy vXPolicy) {
 		
 		RESTResponse ret = null;
@@ -709,7 +727,8 @@ public class AssetREST {
 	
 	@POST
 	@Path("/resources/revoke")
-	@Produces({ "application/json", "application/xml" })
+	@Consumes({ "application/json" })
+	@Produces({ "application/json" })
 	public VXPolicy revokePermission(@Context HttpServletRequest request,VXPolicy vXPolicy) {
 		
 		RESTResponse ret = null;
@@ -742,7 +761,7 @@ public class AssetREST {
 
 	@GET
 	@Path("/ugsyncAudits")
-	@Produces({ "application/json", "application/xml" })
+	@Produces({ "application/json" })
 	@PreAuthorize("@rangerPreAuthSecurityHandler.isAPIAccessible(\"" + RangerAPIList.GET_UGSYNC_AUDITS + "\")")
 	public VXUgsyncAuditInfoList getUgsyncAudits(@Context HttpServletRequest request){
 
@@ -769,7 +788,7 @@ public class AssetREST {
 	@GET
 	@Path("/ugsyncAudits/{syncSource}")
 	@Encoded
-	@Produces({ "application/json", "application/xml" })
+	@Produces({ "application/json" })
 	@PreAuthorize("@rangerPreAuthSecurityHandler.isAPIAccessible(\"" + RangerAPIList.GET_UGSYNC_AUDITS_BY_SYNCSOURCE + "\")")
 	public VXUgsyncAuditInfoList getUgsyncAuditsBySyncSource(@PathParam("syncSource") String syncSource){
 		VXUgsyncAuditInfoList vxUgsyncAuditInfoList = new VXUgsyncAuditInfoList();
