@@ -25,16 +25,19 @@ import java.util.HashSet;
 import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
+import java.util.Objects;
 import java.util.Set;
 
 import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpServletResponse;
+import javax.ws.rs.Consumes;
 import javax.ws.rs.DELETE;
 import javax.ws.rs.GET;
 import javax.ws.rs.POST;
 import javax.ws.rs.PUT;
 import javax.ws.rs.Path;
 import javax.ws.rs.PathParam;
+import javax.ws.rs.Produces;
 import javax.ws.rs.WebApplicationException;
 import javax.ws.rs.core.Context;
 
@@ -109,6 +112,8 @@ public class SecurityZoneREST {
 
     @POST
     @Path("/zones")
+    @Consumes({ "application/json" })
+    @Produces({ "application/json" })
     public RangerSecurityZone createSecurityZone(RangerSecurityZone securityZone) {
         if (LOG.isDebugEnabled()) {
             LOG.debug("==> createSecurityZone("+ securityZone + ")");
@@ -136,6 +141,8 @@ public class SecurityZoneREST {
 
     @PUT
     @Path("/zones/{id}")
+    @Consumes({ "application/json" })
+    @Produces({ "application/json" })
     public RangerSecurityZone updateSecurityZone(@PathParam("id") Long zoneId,
                                                  RangerSecurityZone securityZone) {
         if (LOG.isDebugEnabled()) {
@@ -222,6 +229,7 @@ public class SecurityZoneREST {
 
     @GET
     @Path("/zones/name/{name}")
+    @Produces({ "application/json" })
     public RangerSecurityZone getSecurityZone(@PathParam("name") String zoneName) {
         if (LOG.isDebugEnabled()) {
             LOG.debug("==> getSecurityZone(name=" + zoneName + ")");
@@ -250,6 +258,7 @@ public class SecurityZoneREST {
 
     @GET
     @Path("/zones/{id}")
+    @Produces({ "application/json" })
     public RangerSecurityZone getSecurityZone(@PathParam("id") Long id) {
         if (LOG.isDebugEnabled()) {
             LOG.debug("==> getSecurityZone(id=" + id + ")");
@@ -282,6 +291,7 @@ public class SecurityZoneREST {
 
     @GET
     @Path("/zones")
+    @Produces({ "application/json" })
     public RangerSecurityZoneList getAllZones(@Context HttpServletRequest request) {
         if (LOG.isDebugEnabled()) {
             LOG.debug("==> getAllZones()");
@@ -339,31 +349,25 @@ public class SecurityZoneREST {
 				/* Validation for non service related fields of security zone */
 				
 				
-				if (!securityZone.getName().equals(
-						existingSecurityZone.getName())) {
+				if (!Objects.equals(securityZone.getName(), existingSecurityZone.getName())) {
 					throwRestError("User : " + userName
 							+ " is not allowed to edit zone name of zone : " + existingSecurityZone.getName());
-				} else if (!securityZone.getDescription().equals(
-						existingSecurityZone.getDescription())) {
+				} else if (!Objects.equals(securityZone.getDescription(), existingSecurityZone.getDescription())) {
 					throwRestError("User : " + userName
 							+ " is not allowed to edit zone description of zone : " + existingSecurityZone.getName());
 				}
 				if (!serviceMgr.isZoneAdmin(existingSecurityZone.getName())) {
-					if (!securityZone.getAdminUserGroups().equals(
-							existingSecurityZone.getAdminUserGroups())) {
+					if (!Objects.equals(securityZone.getAdminUserGroups(), existingSecurityZone.getAdminUserGroups())) {
 						throwRestError("User : "
 								+ userName
 								+ " is not allowed to edit zone Admin User Group of zone : " + existingSecurityZone.getName());
-					} else if (!securityZone.getAdminUsers().equals(
-							existingSecurityZone.getAdminUsers())) {
+					} else if (!Objects.equals(securityZone.getAdminUsers(), existingSecurityZone.getAdminUsers())) {
 						throwRestError("User : " + userName
 								+ " is not allowed to edit zone Admin User of zone : " + existingSecurityZone.getName());
-					} else if (!securityZone.getAuditUsers().equals(
-							existingSecurityZone.getAuditUsers())) {
+					} else if (!Objects.equals(securityZone.getAuditUsers(), existingSecurityZone.getAuditUsers())) {
 						throwRestError("User : " + userName
 								+ " is not allowed to edit zone Audit User of zone : " + existingSecurityZone.getName());
-					} else if (!securityZone.getAuditUserGroups().equals(
-							existingSecurityZone.getAuditUserGroups())) {
+					} else if (!Objects.equals(securityZone.getAuditUserGroups(), existingSecurityZone.getAuditUserGroups())) {
 						throwRestError("User : "
 								+ userName
 								+ " is not allowed to edit zone Audit User Group of zone : " + existingSecurityZone.getName());
@@ -449,8 +453,7 @@ public class SecurityZoneREST {
 							.getServices().get(svc);
 
 					if (rangerSecurityZnSvcFromUI != null) {
-						if (!rangerSecurityZnSvcFromDB.getResources().equals(
-								rangerSecurityZnSvcFromUI.getResources())) {
+						if (!Objects.equals(rangerSecurityZnSvcFromDB.getResources(), rangerSecurityZnSvcFromUI.getResources())) {
 							if (!svcStore.isServiceAdminUser(svc, userName)) {
 								throwRestError("User : "
 										+ userName
