@@ -19,12 +19,14 @@
 
 package org.apache.ranger.rest;
 
+import javax.ws.rs.Consumes;
 import org.apache.commons.collections.CollectionUtils;
 import org.apache.commons.lang.StringUtils;
 import org.apache.ranger.biz.AssetMgr;
 import org.apache.ranger.biz.RangerBizUtil;
 import org.apache.ranger.biz.ServiceDBStore;
 import org.apache.ranger.biz.TagDBStore;
+import org.apache.ranger.common.MessageEnums;
 import org.apache.ranger.common.RESTErrorUtil;
 import org.apache.ranger.db.RangerDaoManager;
 import org.apache.ranger.entity.XXService;
@@ -38,6 +40,7 @@ import org.apache.ranger.plugin.model.RangerTagDef;
 import org.apache.ranger.plugin.store.EmbeddedServiceDefsUtil;
 import org.apache.ranger.plugin.store.TagStore;
 import org.apache.ranger.plugin.store.TagValidator;
+import org.apache.ranger.plugin.util.RangerPerfTracer;
 import org.apache.ranger.plugin.util.RangerRESTUtils;
 import org.apache.ranger.plugin.util.SearchFilter;
 import org.apache.ranger.plugin.util.ServiceTags;
@@ -72,8 +75,9 @@ import java.util.List;
 @Scope("request")
 @Transactional(propagation = Propagation.REQUIRES_NEW)
 public class TagREST {
+    private static final Logger LOG      = LoggerFactory.getLogger(TagREST.class);
+    private static final Logger PERF_LOG = RangerPerfTracer.getPerfLogger("rest.TagREST");
 
-    private static final Logger LOG = LoggerFactory.getLogger(TagREST.class);
     public static final String Allowed_User_List_For_Tag_Download = "tag.download.auth.users";
 
 	@Autowired
@@ -113,7 +117,8 @@ public class TagREST {
 
     @POST
     @Path(TagRESTConstants.TAGDEFS_RESOURCE)
-    @Produces({ "application/json", "application/xml" })
+    @Consumes({ "application/json" })
+    @Produces({ "application/json" })
     @PreAuthorize("hasRole('ROLE_SYS_ADMIN')")
     public RangerTagDef createTagDef(RangerTagDef tagDef, @DefaultValue("true") @QueryParam("updateIfExists") boolean updateIfExists) {
         if(LOG.isDebugEnabled()) {
@@ -147,7 +152,8 @@ public class TagREST {
 
     @PUT
     @Path(TagRESTConstants.TAGDEF_RESOURCE + "{id}")
-    @Produces({ "application/json", "application/xml" })
+    @Consumes({ "application/json" })
+    @Produces({ "application/json" })
     @PreAuthorize("hasRole('ROLE_SYS_ADMIN')")
     public RangerTagDef updateTagDef(@PathParam("id") Long id, RangerTagDef tagDef) {
 
@@ -177,7 +183,6 @@ public class TagREST {
 
     @DELETE
     @Path(TagRESTConstants.TAGDEF_RESOURCE + "{id}")
-    @Produces({ "application/json", "application/xml" })
     @PreAuthorize("hasRole('ROLE_SYS_ADMIN')")
     public void deleteTagDef(@PathParam("id") Long id) {
         if(LOG.isDebugEnabled()) {
@@ -199,7 +204,6 @@ public class TagREST {
 
     @DELETE
     @Path(TagRESTConstants.TAGDEF_RESOURCE + "guid/{guid}")
-    @Produces({ "application/json", "application/xml" })
     @PreAuthorize("hasRole('ROLE_SYS_ADMIN')")
     public void deleteTagDefByGuid(@PathParam("guid") String guid) {
         if(LOG.isDebugEnabled()) {
@@ -224,7 +228,7 @@ public class TagREST {
 
     @GET
     @Path(TagRESTConstants.TAGDEF_RESOURCE + "{id}")
-    @Produces({ "application/json", "application/xml" })
+    @Produces({ "application/json" })
     @PreAuthorize("hasRole('ROLE_SYS_ADMIN')")
     public RangerTagDef getTagDef(@PathParam("id") Long id) {
         if(LOG.isDebugEnabled()) {
@@ -254,7 +258,7 @@ public class TagREST {
 
     @GET
     @Path(TagRESTConstants.TAGDEF_RESOURCE + "guid/{guid}")
-    @Produces({ "application/json", "application/xml" })
+    @Produces({ "application/json" })
     @PreAuthorize("hasRole('ROLE_SYS_ADMIN')")
     public RangerTagDef getTagDefByGuid(@PathParam("guid") String guid) {
         if(LOG.isDebugEnabled()) {
@@ -284,7 +288,7 @@ public class TagREST {
 
     @GET
     @Path(TagRESTConstants.TAGDEF_RESOURCE + "name/{name}")
-    @Produces({ "application/json", "application/xml" })
+    @Produces({ "application/json" })
     @PreAuthorize("hasRole('ROLE_SYS_ADMIN')")
     public RangerTagDef getTagDefByName(@PathParam("name") String name) {
         if(LOG.isDebugEnabled()) {
@@ -314,7 +318,7 @@ public class TagREST {
 
     @GET
     @Path(TagRESTConstants.TAGDEFS_RESOURCE)
-    @Produces({ "application/json", "application/xml" })
+    @Produces({ "application/json" })
     @PreAuthorize("hasRole('ROLE_SYS_ADMIN')")
     public List<RangerTagDef> getAllTagDefs() {
         if(LOG.isDebugEnabled()) {
@@ -344,7 +348,7 @@ public class TagREST {
 
     @GET
     @Path(TagRESTConstants.TAGTYPES_RESOURCE)
-    @Produces({ "application/json", "application/xml" })
+    @Produces({ "application/json" })
     @PreAuthorize("hasRole('ROLE_SYS_ADMIN')")
     public List<String> getTagTypes() {
         if(LOG.isDebugEnabled()) {
@@ -371,7 +375,8 @@ public class TagREST {
 
     @POST
     @Path(TagRESTConstants.TAGS_RESOURCE)
-    @Produces({ "application/json", "application/xml" })
+    @Consumes({ "application/json" })
+    @Produces({ "application/json" })
     @PreAuthorize("hasRole('ROLE_SYS_ADMIN')")
     public RangerTag createTag(RangerTag tag, @DefaultValue("true") @QueryParam("updateIfExists") boolean updateIfExists) {
         if(LOG.isDebugEnabled()) {
@@ -404,7 +409,8 @@ public class TagREST {
 
     @PUT
     @Path(TagRESTConstants.TAG_RESOURCE + "{id}")
-    @Produces({ "application/json", "application/xml" })
+    @Consumes({ "application/json" })
+    @Produces({ "application/json" })
     @PreAuthorize("hasRole('ROLE_SYS_ADMIN')")
     public RangerTag updateTag(@PathParam("id") Long id, RangerTag tag) {
 
@@ -426,7 +432,8 @@ public class TagREST {
 
     @PUT
     @Path(TagRESTConstants.TAG_RESOURCE + "guid/{guid}")
-    @Produces({ "application/json", "application/xml" })
+    @Consumes({ "application/json" })
+    @Produces({ "application/json" })
     @PreAuthorize("hasRole('ROLE_SYS_ADMIN')")
     public RangerTag updateTagByGuid(@PathParam("guid") String guid, RangerTag tag) {
 
@@ -452,7 +459,6 @@ public class TagREST {
 
     @DELETE
     @Path(TagRESTConstants.TAG_RESOURCE + "{id}")
-    @Produces({ "application/json", "application/xml" })
     @PreAuthorize("hasRole('ROLE_SYS_ADMIN')")
     public void deleteTag(@PathParam("id") Long id) {
         if(LOG.isDebugEnabled()) {
@@ -474,7 +480,6 @@ public class TagREST {
 
     @DELETE
     @Path(TagRESTConstants.TAG_RESOURCE + "guid/{guid}")
-    @Produces({ "application/json", "application/xml" })
     @PreAuthorize("hasRole('ROLE_SYS_ADMIN')")
     public void deleteTagByGuid(@PathParam("guid") String guid) {
         if(LOG.isDebugEnabled()) {
@@ -497,7 +502,7 @@ public class TagREST {
 
     @GET
     @Path(TagRESTConstants.TAG_RESOURCE + "{id}")
-    @Produces({ "application/json", "application/xml" })
+    @Produces({ "application/json" })
     @PreAuthorize("hasRole('ROLE_SYS_ADMIN')")
     public RangerTag getTag(@PathParam("id") Long id) {
         if(LOG.isDebugEnabled()) {
@@ -522,7 +527,7 @@ public class TagREST {
 
     @GET
     @Path(TagRESTConstants.TAG_RESOURCE + "guid/{guid}")
-    @Produces({ "application/json", "application/xml" })
+    @Produces({ "application/json" })
     @PreAuthorize("hasRole('ROLE_SYS_ADMIN')")
     public RangerTag getTagByGuid(@PathParam("guid") String guid) {
         if(LOG.isDebugEnabled()) {
@@ -547,7 +552,7 @@ public class TagREST {
 
     @GET
     @Path(TagRESTConstants.TAGS_RESOURCE + "type/{type}")
-    @Produces({ "application/json", "application/xml" })
+    @Produces({ "application/json" })
     @PreAuthorize("hasRole('ROLE_SYS_ADMIN')")
     public List<RangerTag> getTagsByType(@PathParam("type") String type) {
         if(LOG.isDebugEnabled()) {
@@ -572,7 +577,7 @@ public class TagREST {
 
     @GET
     @Path(TagRESTConstants.TAGS_RESOURCE)
-    @Produces({ "application/json", "application/xml" })
+    @Produces({ "application/json" })
     @PreAuthorize("hasRole('ROLE_SYS_ADMIN')")
     public List<RangerTag> getAllTags() {
         if(LOG.isDebugEnabled()) {
@@ -601,9 +606,89 @@ public class TagREST {
         return ret;
     }
 
+    /**
+     * Resets/ removes tag policy cache for given service.
+     * @param serviceName non-empty service-name
+     * @return {@code true} if successfully reseted/ removed for given service, {@code false} otherwise.
+     */
+    @GET
+    @Path(TagRESTConstants.TAGS_RESOURCE + "cache/reset")
+    @Produces({ "application/json" })
+    public boolean resetTagCache(@QueryParam("serviceName") String serviceName) {
+        if (LOG.isDebugEnabled()) {
+            LOG.debug("==> TagREST.resetTagCache({})", serviceName);
+        }
+
+        if (StringUtils.isEmpty(serviceName)) {
+            throw restErrorUtil.createRESTException("Required parameter [serviceName] is missing.", MessageEnums.INVALID_INPUT_DATA);
+        }
+
+        RangerService rangerService = null;
+        try {
+            rangerService = svcStore.getServiceByName(serviceName);
+        } catch (Exception e) {
+            LOG.error( HttpServletResponse.SC_BAD_REQUEST + "No Service Found for ServiceName:" + serviceName );
+        }
+
+        if (rangerService == null) {
+            throw restErrorUtil.createRESTException(HttpServletResponse.SC_BAD_REQUEST , "Invalid service name", true);
+        }
+
+        // check for ADMIN access
+        if (!bizUtil.isAdmin()) {
+            boolean isServiceAdmin = false;
+            String  loggedInUser   = bizUtil.getCurrentUserLoginId();
+
+            try {
+                isServiceAdmin = bizUtil.isUserServiceAdmin(rangerService, loggedInUser);
+            } catch (Exception e) {
+                LOG.warn("Failed to find if user [" + loggedInUser + "] has service admin privileges on service [" + serviceName + "]", e);
+            }
+
+            if (!isServiceAdmin) {
+                throw restErrorUtil.createRESTException("User cannot reset tag cache", MessageEnums.OPER_NO_PERMISSION);
+            }
+        }
+
+        boolean ret = tagStore.resetTagCache(serviceName);
+
+        if (LOG.isDebugEnabled()) {
+            LOG.debug("<== TagREST.resetTagCache(): ret={}", ret);
+        }
+
+        return ret;
+    }
+
+    /**
+     * Resets/ removes tag policy cache for all.
+     * @return {@code true} if successfully reseted/ removed, {@code false} otherwise.
+     */
+    @GET
+    @Path(TagRESTConstants.TAGS_RESOURCE + "cache/reset-all")
+    @Produces({ "application/json" })
+    public boolean resetTagCacheAll() {
+        if (LOG.isDebugEnabled()) {
+            LOG.debug("==> TagREST.resetTagCacheAll()");
+        }
+
+        // check for ADMIN access
+        if (!bizUtil.isAdmin()) {
+            throw restErrorUtil.createRESTException("User cannot reset policy cache", MessageEnums.OPER_NO_PERMISSION);
+        }
+
+        boolean ret = tagStore.resetTagCache(null);
+
+        if (LOG.isDebugEnabled()) {
+            LOG.debug("<== TagREST.resetTagCacheAll(): ret={}", ret);
+        }
+
+        return ret;
+    }
+
     @POST
     @Path(TagRESTConstants.RESOURCES_RESOURCE)
-    @Produces({ "application/json", "application/xml" })
+    @Consumes({ "application/json" })
+    @Produces({ "application/json" })
     @PreAuthorize("hasRole('ROLE_SYS_ADMIN')")
     public RangerServiceResource createServiceResource(RangerServiceResource resource, @DefaultValue("true") @QueryParam("updateIfExists") boolean updateIfExists) {
         if(LOG.isDebugEnabled()) {
@@ -636,7 +721,8 @@ public class TagREST {
 
     @PUT
     @Path(TagRESTConstants.RESOURCE_RESOURCE + "{id}")
-    @Produces({ "application/json", "application/xml" })
+    @Consumes({ "application/json" })
+    @Produces({ "application/json" })
     @PreAuthorize("hasRole('ROLE_SYS_ADMIN')")
     public RangerServiceResource updateServiceResource(@PathParam("id") Long id, RangerServiceResource resource) {
         if(LOG.isDebugEnabled()) {
@@ -661,7 +747,8 @@ public class TagREST {
 
     @PUT
     @Path(TagRESTConstants.RESOURCE_RESOURCE + "guid/{guid}")
-    @Produces({ "application/json", "application/xml" })
+    @Consumes({ "application/json" })
+    @Produces({ "application/json" })
     @PreAuthorize("hasRole('ROLE_SYS_ADMIN')")
     public RangerServiceResource updateServiceResourceByGuid(@PathParam("guid") String guid, RangerServiceResource resource) {
         if(LOG.isDebugEnabled()) {
@@ -684,7 +771,6 @@ public class TagREST {
 
     @DELETE
     @Path(TagRESTConstants.RESOURCE_RESOURCE + "{id}")
-    @Produces({ "application/json", "application/xml" })
     @PreAuthorize("hasRole('ROLE_SYS_ADMIN')")
     public void deleteServiceResource(@PathParam("id") Long id) {
         if(LOG.isDebugEnabled()) {
@@ -706,7 +792,6 @@ public class TagREST {
 
     @DELETE
     @Path(TagRESTConstants.RESOURCE_RESOURCE + "guid/{guid}")
-    @Produces({ "application/json", "application/xml" })
     @PreAuthorize("hasRole('ROLE_SYS_ADMIN')")
     public void deleteServiceResourceByGuid(@PathParam("guid") String guid, @DefaultValue("false") @QueryParam("deleteReferences") boolean deleteReferences) {
         if(LOG.isDebugEnabled()) {
@@ -737,7 +822,7 @@ public class TagREST {
 
     @GET
     @Path(TagRESTConstants.RESOURCE_RESOURCE + "{id}")
-    @Produces({ "application/json", "application/xml" })
+    @Produces({ "application/json" })
     @PreAuthorize("hasRole('ROLE_SYS_ADMIN')")
     public RangerServiceResource getServiceResource(@PathParam("id") Long id) {
         if(LOG.isDebugEnabled()) {
@@ -759,7 +844,7 @@ public class TagREST {
 
     @GET
     @Path(TagRESTConstants.RESOURCE_RESOURCE + "guid/{guid}")
-    @Produces({ "application/json", "application/xml" })
+    @Produces({ "application/json" })
     @PreAuthorize("hasRole('ROLE_SYS_ADMIN')")
     public RangerServiceResource getServiceResourceByGuid(@PathParam("guid") String guid) {
         if(LOG.isDebugEnabled()) {
@@ -781,7 +866,7 @@ public class TagREST {
 
     @GET
     @Path(TagRESTConstants.RESOURCES_RESOURCE + "service/{serviceName}")
-    @Produces({ "application/json", "application/xml" })
+    @Produces({ "application/json" })
     @PreAuthorize("hasRole('ROLE_SYS_ADMIN')")
     public List<RangerServiceResource> getServiceResourcesByService(@PathParam("serviceName") String serviceName) {
         if(LOG.isDebugEnabled()) {
@@ -812,7 +897,7 @@ public class TagREST {
 
     @GET
     @Path(TagRESTConstants.RESOURCE_RESOURCE + "service/{serviceName}/signature/{resourceSignature}")
-    @Produces({ "application/json", "application/xml" })
+    @Produces({ "application/json" })
     @PreAuthorize("hasRole('ROLE_SYS_ADMIN')")
     public RangerServiceResource getServiceResourceByServiceAndResourceSignature(@PathParam("serviceName") String serviceName,
                                                                        @PathParam("resourceSignature") String resourceSignature) {
@@ -839,7 +924,7 @@ public class TagREST {
 
     @GET
     @Path(TagRESTConstants.RESOURCES_RESOURCE)
-    @Produces({ "application/json", "application/xml" })
+    @Produces({ "application/json" })
     @PreAuthorize("hasRole('ROLE_SYS_ADMIN')")
     public List<RangerServiceResource> getAllServiceResources() {
         if(LOG.isDebugEnabled()) {
@@ -865,7 +950,8 @@ public class TagREST {
 
     @POST
     @Path(TagRESTConstants.TAGRESOURCEMAPS_RESOURCE)
-    @Produces({ "application/json", "application/xml" })
+    @Consumes({ "application/json" })
+    @Produces({ "application/json" })
     @PreAuthorize("hasRole('ROLE_SYS_ADMIN')")
     public RangerTagResourceMap createTagResourceMap(@QueryParam("tag-guid") String tagGuid, @QueryParam("resource-guid") String resourceGuid,
                                                      @DefaultValue("false") @QueryParam("lenient") boolean lenient) {
@@ -899,7 +985,6 @@ public class TagREST {
 
     @DELETE
     @Path(TagRESTConstants.TAGRESOURCEMAP_RESOURCE + "{id}")
-    @Produces({ "application/json", "application/xml" })
     @PreAuthorize("hasRole('ROLE_SYS_ADMIN')")
     public void deleteTagResourceMap(@PathParam("id") Long id) {
         if(LOG.isDebugEnabled()) {
@@ -921,7 +1006,6 @@ public class TagREST {
 
     @DELETE
     @Path(TagRESTConstants.TAGRESOURCEMAP_RESOURCE + "guid/{guid}")
-    @Produces({ "application/json", "application/xml" })
     @PreAuthorize("hasRole('ROLE_SYS_ADMIN')")
     public void deleteTagResourceMapByGuid(@PathParam("guid") String guid) {
         if(LOG.isDebugEnabled()) {
@@ -930,7 +1014,7 @@ public class TagREST {
 
         try {
             RangerTagResourceMap exist = validator.preDeleteTagResourceMapByGuid(guid);
-            tagStore.deleteServiceResource(exist.getId());
+            tagStore.deleteTagResourceMap(exist.getId());
         } catch(Exception excp) {
             LOG.error("deleteTagResourceMapByGuid(" + guid + ") failed", excp);
 
@@ -944,7 +1028,6 @@ public class TagREST {
 
     @DELETE
     @Path(TagRESTConstants.TAGRESOURCEMAPS_RESOURCE)
-    @Produces({ "application/json", "application/xml" })
     @PreAuthorize("hasRole('ROLE_SYS_ADMIN')")
     public void deleteTagResourceMap(@QueryParam("tag-guid") String tagGuid, @QueryParam("resource-guid") String resourceGuid) {
         if(LOG.isDebugEnabled()) {
@@ -967,7 +1050,7 @@ public class TagREST {
 
     @GET
     @Path(TagRESTConstants.TAGRESOURCEMAP_RESOURCE + "{id}")
-    @Produces({ "application/json", "application/xml" })
+    @Produces({ "application/json" })
     @PreAuthorize("hasRole('ROLE_SYS_ADMIN')")
     public RangerTagResourceMap getTagResourceMap(@PathParam("id") Long id) {
         if(LOG.isDebugEnabled()) {
@@ -992,7 +1075,7 @@ public class TagREST {
 
     @GET
     @Path(TagRESTConstants.TAGRESOURCEMAP_RESOURCE + "guid/{guid}")
-    @Produces({ "application/json", "application/xml" })
+    @Produces({ "application/json" })
     @PreAuthorize("hasRole('ROLE_SYS_ADMIN')")
     public RangerTagResourceMap getTagResourceMapByGuid(@PathParam("guid") String guid) {
         if(LOG.isDebugEnabled()) {
@@ -1017,7 +1100,7 @@ public class TagREST {
 
     @GET
     @Path(TagRESTConstants.TAGRESOURCEMAP_RESOURCE + "tag-resource-guid")
-    @Produces({ "application/json", "application/xml" })
+    @Produces({ "application/json" })
     @PreAuthorize("hasRole('ROLE_SYS_ADMIN')")
     public RangerTagResourceMap getTagResourceMap(@QueryParam("tagGuid") String tagGuid, @QueryParam("resourceGuid") String resourceGuid) {
         if(LOG.isDebugEnabled()) {
@@ -1043,7 +1126,7 @@ public class TagREST {
 
     @GET
     @Path(TagRESTConstants.TAGRESOURCEMAPS_RESOURCE)
-    @Produces({ "application/json", "application/xml" })
+    @Produces({ "application/json" })
     @PreAuthorize("hasRole('ROLE_SYS_ADMIN')")
     public List<RangerTagResourceMap> getAllTagResourceMaps() {
         if(LOG.isDebugEnabled()) {
@@ -1076,24 +1159,31 @@ public class TagREST {
 
     @PUT
     @Path(TagRESTConstants.IMPORT_SERVICETAGS_RESOURCE)
-    @Produces({ "application/json", "application/xml" })
+    @Consumes({ "application/json" })
+    @Produces({ "application/json" })
     @PreAuthorize("hasRole('ROLE_SYS_ADMIN')")
     public void importServiceTags(ServiceTags serviceTags) {
         if(LOG.isDebugEnabled()) {
             LOG.debug("==> TagREST.importServiceTags()");
         }
 
-        try {
+        RangerPerfTracer perf = null;
 
+        if(RangerPerfTracer.isPerfTraceEnabled(PERF_LOG)) {
+            perf = RangerPerfTracer.getPerfTracer(PERF_LOG, "TagREST.importServiceTags(service=" + (serviceTags != null ? serviceTags.getServiceName() : null) + ")");
+        }
+
+        try {
             ServiceTagsProcessor serviceTagsProcessor = new ServiceTagsProcessor(tagStore);
             serviceTagsProcessor.process(serviceTags);
-
         } catch (Exception excp) {
             LOG.error("importServiceTags() failed", excp);
 
             throw restErrorUtil.createRESTException(HttpServletResponse.SC_BAD_REQUEST, excp.getMessage(), true);
-
+        } finally {
+            RangerPerfTracer.log(perf);
         }
+
         if(LOG.isDebugEnabled()) {
             LOG.debug("<== TagREST.importServiceTags()");
         }
@@ -1103,7 +1193,7 @@ public class TagREST {
 
     @GET
     @Path(TagRESTConstants.TAGS_DOWNLOAD + "{serviceName}")
-    @Produces({ "application/json", "application/xml" })
+    @Produces({ "application/json" })
     public ServiceTags getServiceTagsIfUpdated(@PathParam("serviceName") String serviceName,
                                                    @QueryParam(TagRESTConstants.LAST_KNOWN_TAG_VERSION_PARAM) Long lastKnownVersion,
                                                @DefaultValue("0") @QueryParam(TagRESTConstants.LAST_ACTIVATION_TIME) Long lastActivationTime, @QueryParam("pluginId") String pluginId,
@@ -1114,6 +1204,12 @@ public class TagREST {
             LOG.debug("==> TagREST.getServiceTagsIfUpdated(" + serviceName + ", " + lastKnownVersion + ", " + lastActivationTime + ", " + pluginId + ", " + supportsTagDeltas + ")");
         }
 
+        RangerPerfTracer perf = null;
+
+        if(RangerPerfTracer.isPerfTraceEnabled(PERF_LOG)) {
+            perf = RangerPerfTracer.getPerfTracer(PERF_LOG, "TagREST.getServiceTagsIfUpdated(service=" + serviceName + ", lastKnownVersion=" + lastKnownVersion + ")");
+        }
+
 		ServiceTags ret      = null;
 		int         httpCode = HttpServletResponse.SC_OK;
 		String      logMsg   = null;
@@ -1124,7 +1220,7 @@ public class TagREST {
 		}
 
         try {
-            bizUtil.failUnauthenticatedIfNotAllowed();
+            bizUtil.failUnauthenticatedDownloadIfNotAllowed();
 
             ret = tagStore.getServiceTagsIfUpdated(serviceName, lastKnownVersion, !supportsTagDeltas);
 
@@ -1145,6 +1241,8 @@ public class TagREST {
 			logMsg   = excp.getMessage();
         } finally {
             assetMgr.createPluginInfo(serviceName, pluginId, request, RangerPluginInfo.ENTITY_TYPE_TAGS, downloadedVersion, lastKnownVersion, lastActivationTime, httpCode, clusterName, pluginCapabilities);
+
+            RangerPerfTracer.log(perf);
         }
 
         if(httpCode != HttpServletResponse.SC_OK) {
@@ -1161,7 +1259,7 @@ public class TagREST {
 
     @GET
     @Path(TagRESTConstants.TAGS_SECURE_DOWNLOAD + "{serviceName}")
-    @Produces({ "application/json", "application/xml" })
+    @Produces({ "application/json" })
     public ServiceTags getSecureServiceTagsIfUpdated(@PathParam("serviceName") String serviceName,
                                                    @QueryParam(TagRESTConstants.LAST_KNOWN_TAG_VERSION_PARAM) Long lastKnownVersion,
                                                      @DefaultValue("0") @QueryParam(TagRESTConstants.LAST_ACTIVATION_TIME) Long lastActivationTime, @QueryParam("pluginId") String pluginId,
@@ -1173,6 +1271,12 @@ public class TagREST {
             LOG.debug("==> TagREST.getSecureServiceTagsIfUpdated(" + serviceName + ", " + lastKnownVersion + ", " + lastActivationTime + ", " + pluginId + ", " + supportsTagDeltas + ")");
         }
 
+        RangerPerfTracer perf = null;
+
+        if(RangerPerfTracer.isPerfTraceEnabled(PERF_LOG)) {
+            perf = RangerPerfTracer.getPerfTracer(PERF_LOG, "TagREST.getSecureServiceTagsIfUpdated(service=" + serviceName + ", lastKnownVersion=" + lastKnownVersion + ")");
+        }
+
 		ServiceTags ret      = null;
 		int         httpCode = HttpServletResponse.SC_OK;
 		String      logMsg   = null;
@@ -1233,6 +1337,8 @@ public class TagREST {
 			logMsg   = excp.getMessage();
         }  finally {
             assetMgr.createPluginInfo(serviceName, pluginId, request, RangerPluginInfo.ENTITY_TYPE_TAGS, downloadedVersion, lastKnownVersion, lastActivationTime, httpCode, clusterName, pluginCapabilities);
+
+            RangerPerfTracer.log(perf);
         }
 
         if(httpCode != HttpServletResponse.SC_OK) {
