@@ -29,21 +29,27 @@ import javax.security.auth.login.AppConfigurationEntry;
 import javax.security.auth.login.AppConfigurationEntry.LoginModuleControlFlag;
 import javax.security.auth.login.Configuration;
 
-import org.apache.log4j.Logger;
 import org.apache.ranger.authentication.unix.jaas.RoleUserAuthorityGranter;
+import org.apache.ranger.authorization.hadoop.config.RangerAdminConfig;
 import org.apache.ranger.authorization.utils.StringUtil;
 import org.apache.ranger.common.PropertiesUtil;
+import org.apache.ranger.util.Pbkdf2PasswordEncoderCust;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
+import org.springframework.context.support.MessageSourceAccessor;
 import org.springframework.ldap.core.support.DefaultTlsDirContextAuthenticationStrategy;
 import org.springframework.ldap.core.support.LdapContextSource;
 import org.springframework.security.authentication.AuthenticationProvider;
 import org.springframework.security.authentication.AuthenticationServiceException;
 import org.springframework.security.authentication.BadCredentialsException;
+import org.springframework.security.authentication.LockedException;
 import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
 import org.springframework.security.authentication.jaas.DefaultJaasAuthenticationProvider;
 import org.springframework.security.authentication.jaas.memory.InMemoryConfiguration;
 import org.springframework.security.core.Authentication;
 import org.springframework.security.core.AuthenticationException;
 import org.springframework.security.core.GrantedAuthority;
+import org.springframework.security.core.SpringSecurityMessageSource;
 import org.springframework.security.core.authority.SimpleGrantedAuthority;
 import org.springframework.security.core.userdetails.User;
 import org.springframework.security.core.userdetails.UserDetails;
@@ -56,12 +62,10 @@ import org.springframework.security.ldap.userdetails.DefaultLdapAuthoritiesPopul
 import org.springframework.security.ldap.search.FilterBasedLdapUserSearch;
 import org.springframework.security.provisioning.JdbcUserDetailsManager;
 import org.springframework.security.authentication.dao.DaoAuthenticationProvider;
-import org.springframework.security.authentication.dao.ReflectionSaltSource;
-import org.springframework.security.authentication.encoding.Md5PasswordEncoder;
-import org.springframework.security.authentication.encoding.ShaPasswordEncoder;
 import org.springframework.beans.factory.annotation.Autowired;
 import org.springframework.beans.factory.annotation.Qualifier;
 import org.apache.ranger.biz.UserMgr;
+import org.apache.ranger.biz.SessionMgr;
 
 
 
@@ -73,14 +77,22 @@ public class RangerAuthenticationProvider implements AuthenticationProvider {
 
 	@Autowired
 	UserMgr userMgr;
-	private static final Logger logger = Logger.getLogger(RangerAuthenticationProvider.class);
+
+	@Autowired
+	SessionMgr sessionMgr;
+
+	private static final Logger logger = LoggerFactory.getLogger(RangerAuthenticationProvider.class);
+
 	private String rangerAuthenticationMethod;
 
 	private LdapAuthenticator authenticator;
 
 	private boolean ssoEnabled = false;
+	private final boolean isFipsEnabled;
+	protected MessageSourceAccessor messages = SpringSecurityMessageSource.getAccessor();
 
 	public RangerAuthenticationProvider() {
+		this.isFipsEnabled = RangerAdminConfig.getInstance().isFipsEnabled();
 
 	}
 
@@ -135,6 +147,23 @@ public class RangerAuthenticationProvider implements AuthenticationProvider {
 					return authentication;
 				}
 			}
+
+			// Following are JDBC
+			if (authentication != null && authentication.getName() != null && sessionMgr.isLoginIdLocked(authentication.getName())) {
+				logger.debug("Failed to authenticate since user account is locked");
+
+				throw new LockedException(messages.getMessage("AbstractUserDetailsAuthenticationProvider.locked", "User account is locked"));
+			}
+
+			if (this.isFipsEnabled) {
+				try {
+					authentication = getJDBCAuthentication(authentication,"");
+				} catch (Exception e) {
+					logger.error("JDBC Authentication failure: ", e);
+					throw e;
+				}
+				return authentication;
+			}
 			String encoder="SHA256";
 			try {
 				authentication = getJDBCAuthentication(authentication,encoder);
@@ -573,19 +602,23 @@ public class RangerAuthenticationProvider implements AuthenticationProvider {
 
 	private Authentication getJDBCAuthentication(Authentication authentication,String encoder) throws AuthenticationException{
 		try {
-
-			ReflectionSaltSource saltSource = new ReflectionSaltSource();
-			saltSource.setUserPropertyToUse("username");
-
 			DaoAuthenticationProvider authenticator = new DaoAuthenticationProvider();
 			authenticator.setUserDetailsService(userDetailsService);
-			if (encoder != null && "SHA256".equalsIgnoreCase(encoder)) {
-				authenticator.setPasswordEncoder(new ShaPasswordEncoder(256));
-			} else if(encoder != null && "MD5".equalsIgnoreCase(encoder)) {
-				authenticator.setPasswordEncoder(new Md5PasswordEncoder());
+			if (this.isFipsEnabled) {
+				if (authentication != null && authentication.getCredentials() != null && !authentication.isAuthenticated()) {
+					Pbkdf2PasswordEncoderCust passwordEncoder = new Pbkdf2PasswordEncoderCust(authentication.getName());
+					passwordEncoder.setEncodeHashAsBase64(true);
+					authenticator.setPasswordEncoder(passwordEncoder);
+				}
+			} else {
+				if (encoder != null && "SHA256".equalsIgnoreCase(encoder) && authentication != null) {
+					authenticator.setPasswordEncoder(new RangerCustomPasswordEncoder(authentication.getName(),"SHA-256"));
+
+				} else if (encoder != null && "MD5".equalsIgnoreCase(encoder)  && authentication != null) {
+					authenticator.setPasswordEncoder(new RangerCustomPasswordEncoder(authentication.getName(),"MD5"));
+				}
 			}
 
-			authenticator.setSaltSource(saltSource);
 			String userName ="";
 			String userPassword = "";
 			if (authentication!=null) {
@@ -616,6 +649,8 @@ public class RangerAuthenticationProvider implements AuthenticationProvider {
 			throw e;
 		}catch (Exception e) {
 			throw e;
+		} catch (Throwable t) {
+			throw new BadCredentialsException("Bad credentials", t);
 		}
 		return authentication;
 	}
