@@ -27,8 +27,6 @@ import java.util.Set;
 
 import org.apache.commons.collections.MapUtils;
 import org.apache.commons.lang.StringUtils;
-import org.apache.commons.logging.Log;
-import org.apache.commons.logging.LogFactory;
 import org.apache.ranger.common.AppConstants;
 import org.apache.ranger.common.GUIDUtil;
 import org.apache.ranger.common.JSONUtil;
@@ -52,11 +50,13 @@ import org.apache.ranger.plugin.model.RangerServiceDef.RangerServiceConfigDef;
 import org.apache.ranger.plugin.util.SearchFilter;
 import org.apache.ranger.plugin.util.ServiceDefUtil;
 import org.apache.ranger.view.RangerServiceDefList;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 import org.springframework.beans.factory.annotation.Autowired;
 
 public abstract class RangerServiceDefServiceBase<T extends XXServiceDefBase, V extends RangerServiceDef>
 		extends RangerBaseModelService<T, V> {
-	private static final Log LOG = LogFactory.getLog(RangerServiceDefServiceBase.class);
+	private static final Logger LOG = LoggerFactory.getLogger(RangerServiceDefServiceBase.class);
 
 	private static final String OPTION_RESOURCE_ACCESS_TYPE_RESTRICTIONS = "__accessTypeRestrictions";
 	private static final String OPTION_RESOURCE_IS_VALID_LEAF            = "__isValidLeaf";
@@ -110,11 +110,12 @@ public abstract class RangerServiceDefServiceBase<T extends XXServiceDefBase, V
 			serviceDef.setResources(resources);
 		}
 
-		List<XXAccessTypeDef> xAccessTypes = daoMgr.getXXAccessTypeDef().findByServiceDefId(serviceDefId);
+		List<XXAccessTypeDef>     xAccessTypes  = daoMgr.getXXAccessTypeDef().findByServiceDefId(serviceDefId);
+		Map<String, List<String>> impliedGrants = daoMgr.getXXAccessTypeDefGrants().findImpliedGrantsByServiceDefId(serviceDefId);
 		if (!stringUtil.isEmpty(xAccessTypes)) {
 			List<RangerAccessTypeDef> accessTypes = new ArrayList<RangerAccessTypeDef>();
 			for (XXAccessTypeDef xAtd : xAccessTypes) {
-				RangerAccessTypeDef accessType = populateXXToRangerAccessTypeDef(xAtd);
+				RangerAccessTypeDef accessType = populateXXToRangerAccessTypeDef(xAtd, impliedGrants.get(xAtd.getName()));
 				accessTypes.add(accessType);
 			}
 			serviceDef.setAccessTypes(accessTypes);
@@ -405,17 +406,27 @@ public abstract class RangerServiceDefServiceBase<T extends XXServiceDefBase, V
 	}
 	
 	public RangerAccessTypeDef populateXXToRangerAccessTypeDef(XXAccessTypeDef xObj) {
+		List<String> impliedGrants = daoMgr.getXXAccessTypeDefGrants().findImpliedGrantsByATDId(xObj.getId());
+
+		return populateXXToRangerAccessTypeDef(xObj, impliedGrants);
+	}
+
+	public RangerAccessTypeDef populateXXToRangerAccessTypeDef(XXAccessTypeDef xObj, List<String> impliedGrants) {
 		RangerAccessTypeDef vObj = new RangerAccessTypeDef();
+
+		if (impliedGrants == null) {
+			impliedGrants = new ArrayList<>();
+		}
+
 		vObj.setItemId(xObj.getItemId());
 		vObj.setName(xObj.getName());
 		vObj.setLabel(xObj.getLabel());
 		vObj.setRbKeyLabel(xObj.getRbkeylabel());
-		
-		List<String> impliedGrants = daoMgr.getXXAccessTypeDefGrants().findImpliedGrantsByATDId(xObj.getId());
 		vObj.setImpliedGrants(impliedGrants);
+
 		return vObj;
 	}
-	
+
 	public XXPolicyConditionDef populateRangerPolicyConditionDefToXX(RangerPolicyConditionDef vObj,
 			XXPolicyConditionDef xObj, XXServiceDef serviceDef, int operationContext) {
 		if(serviceDef == null) {
@@ -582,9 +593,10 @@ public abstract class RangerServiceDefServiceBase<T extends XXServiceDefBase, V
 		RangerServiceDefList retList = new RangerServiceDefList();
 		int startIndex = searchFilter.getStartIndex();
 		int pageSize = searchFilter.getMaxRows();
+		String denyCondition = searchFilter.getParam(SearchFilter.FETCH_DENY_CONDITION);
 		searchFilter.setStartIndex(0);
 		searchFilter.setMaxRows(Integer.MAX_VALUE);
-	
+
 		boolean isAuditPage=false;
 		if(searchFilter.getParam("pageSource")!=null){
 			isAuditPage=true;
@@ -593,7 +605,7 @@ public abstract class RangerServiceDefServiceBase<T extends XXServiceDefBase, V
 				retList);
 		List<T> permittedServiceDefs = new ArrayList<T>();
 		for (T xSvcDef : xSvcDefList) {
-			if (bizUtil.hasAccess(xSvcDef, null) || (bizUtil.isAdmin() && isAuditPage) ) {
+			if ((bizUtil.hasAccess(xSvcDef, null) || (bizUtil.isAdmin() && isAuditPage)) || ("true".equals(denyCondition))) {
 				permittedServiceDefs.add(xSvcDef);
 			}
 		}
