@@ -29,7 +29,6 @@ import java.util.Map.Entry;
 import java.util.Random;
 
 import org.apache.commons.lang.StringUtils;
-import org.apache.log4j.Logger;
 import org.apache.ranger.common.AppConstants;
 import org.apache.ranger.common.DateUtil;
 import org.apache.ranger.common.MessageEnums;
@@ -53,12 +52,14 @@ import org.apache.ranger.view.VXPolicy;
 import org.apache.ranger.view.VXPolicyList;
 import org.apache.ranger.view.VXResource;
 import org.apache.ranger.view.VXResourceList;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 import org.springframework.beans.factory.annotation.Autowired;
 import org.springframework.stereotype.Service;
 
 @Service
 public class XPolicyService extends PublicAPIServiceBase<VXResource, VXPolicy> {
-	Logger logger = Logger.getLogger(XPolicyService.class);
+	Logger logger = LoggerFactory.getLogger(XPolicyService.class);
 
 	@Autowired
 	RESTErrorUtil restErrorUtil;
