@@ -28,15 +28,16 @@ import java.util.HashMap;
 import java.util.Map;
 import java.util.List;
 
-import org.apache.log4j.Logger;
 import org.apache.ranger.common.view.VEnum;
 import org.apache.ranger.common.view.VEnumElement;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 import org.springframework.stereotype.Component;
 
 @Component
 public class RangerEnumUtil {
 
-    private static final Logger logger = Logger.getLogger(RangerEnumUtil.class);
+    private static final Logger logger = LoggerFactory.getLogger(RangerEnumUtil.class);
     public final static String ENUM_CommonEnums_ActiveStatus = "CommonEnums.ActiveStatus";
     public final static String ENUM_CommonEnums_ActivationStatus = "CommonEnums.ActivationStatus";
     public final static String ENUM_CommonEnums_BooleanValue = "CommonEnums.BooleanValue";
@@ -1962,6 +1963,32 @@ public class RangerEnumUtil {
 
 	vEnum.getElementList().add(vElement);
 
+	vElement = new VEnumElement();
+	vElement.setElementName("AUTH_TYPE_KERBEROS");
+	vElement.setElementValue(2);
+	vElement.setElementLabel("Kerberos");
+	vElement.setRbKey("xa.enum.AuthType.AUTH_TYPE_KERBEROS");
+	vElement.setEnumName(vEnum.getEnumName());
+
+	vEnum.getElementList().add(vElement);
+
+	vElement = new VEnumElement();
+	vElement.setElementName("AUTH_TYPE_SSO");
+	vElement.setElementValue(3);
+	vElement.setElementLabel("SingleSignOn");
+	vElement.setRbKey("xa.enum.AuthType.AUTH_TYPE_SSO");
+	vElement.setEnumName(vEnum.getEnumName());
+
+	vEnum.getElementList().add(vElement);
+
+	vElement = new VEnumElement();
+	vElement.setElementName("AUTH_TYPE_TRUSTED_PROXY");
+	vElement.setElementValue(4);
+	vElement.setElementLabel("Trusted Proxy");
+	vElement.setRbKey("xa.enum.AuthType.AUTH_TYPE_TRUSTED_PROXY");
+	vElement.setEnumName(vEnum.getEnumName());
+
+	vEnum.getElementList().add(vElement);
 
 	///////////////////////////////////
 	// XResponse::ResponseStatus
