@@ -145,6 +145,11 @@ public class VXResource extends VXDataObject implements java.io.Serializable {
 	 */
 	protected String services;
 
+	/**
+	 * Hive Services
+	 */
+	protected String hiveServices;
+
 	/**
 	 * guid
 	 */
@@ -565,6 +570,27 @@ public class VXResource extends VXDataObject implements java.io.Serializable {
 		this.services = services;
 	}
 
+
+	/**
+	 * This method sets the value to the member attribute <b>hiveservices</b>. You
+	 * cannot set null to the attribute.
+	 *
+	 * @param hiveServices
+	 *            Value to set member attribute <b>hiveservices</b>
+	 */
+	public void setHiveServices(String hiveServices) {
+		this.hiveServices = hiveServices;
+	}
+
+	/**
+	 * Returns the value for the member attribute <b>hiveservices</b>
+	 *
+	 * @return String - value of member attribute <b>hiveservices</b>.
+	 */
+	public String getHiveServices() {
+		return hiveServices;
+	}
+
 	/**
 	 * This method sets the value to the member attribute <b>checkParentPermission</b>.
 	 * You cannot set null to the attribute.
