@@ -74,6 +74,7 @@ define(function(require){
 		onRender: function() {
 			this.initializePlugins();
 			$('#r_breadcrumbs').hide();
+			$('.latestResponse').hide();
 			var msg = '', moreInfo = '';
 			if(this.status == 401){
 				msg = 'Access Denied (401)';
@@ -108,6 +109,7 @@ define(function(require){
 		/** on close */
 		onClose: function(){
 			$('#r_breadcrumbs').show();
+			$('.latestResponse').show();
 		}
 
 	});
