@@ -49,6 +49,13 @@ define(function(require){
     	template: RangerPolicyTableLayoutTmpl,
 
 		templateHelpers : function(){
+			var infoMsg ="", displayClass = "d-none";
+			if(this.rangerService && this.rangerService.get('type')){
+				if(this.rangerService.get('type') == XAEnums.ServiceType.Service_HDFS.label || this.rangerService.get('type') == XAEnums.ServiceType.Service_YARN.label) {
+					infoMsg = XAUtil.pluginConfigInfo(this.rangerService.get('type').toUpperCase())
+					displayClass = "show"
+				}
+			}
 			return {
 				rangerService : this.rangerService,
 				rangerServiceDef : this.rangerServiceDefModel,
@@ -57,6 +64,8 @@ define(function(require){
                                         : XAUtil.isRenderRowFilter(this.rangerServiceDefModel.get('rowFilterDef')) ? true : false,
                 isAddNewPolicyButtonShow : !(XAUtil.isAuditorOrKMSAuditor(SessionMgr)) && this.rangerService.get('isEnabled'),
                 setNewUi : localStorage.getItem('setOldUI') == "true" ? false : true,
+                displayClass : displayClass,
+                infoMsg : infoMsg,
 			};
 		},
         
