@@ -40,6 +40,7 @@ define(function(require){
         var VXRole			= require('models/VXRole');
 
 	var UsertablelayoutTmpl = require('hbs!tmpl/users/UserTableLayout_tmpl');
+	var XAViewUtils		= require('utils/XAViewUtils');
 
 	var UserTableLayout = Backbone.Marionette.Layout.extend(
 	/** @lends UserTableLayout */
@@ -76,7 +77,8 @@ define(function(require){
             deleteUser: '[data-id="deleteUserGroup"]',
             showUserList:'[data-js="showUserList"]',
             addNewRoles: '[data-id="addNewRoles"]',
-            hideShowVisibility: '[data-id="hideShowVisibility"]'
+            hideShowVisibility: '[data-id="hideShowVisibility"]',
+            syncDetailes : '[data-id="syncDetailes"]',
     	},
 
 		/** ui events hash */
@@ -86,9 +88,10 @@ define(function(require){
 			events['click ' + this.ui.btnShowMore]  = 'onShowMore';
 			events['click ' + this.ui.btnShowLess]  = 'onShowLess';
 			events['click ' + this.ui.btnSave]  = 'onSave';
-			events['click ' + this.ui.visibilityDropdown +' li a']  = 'onVisibilityChange';
+			events['click ' + this.ui.visibilityDropdown+ ' a']  = 'onVisibilityChange';
 			events['click ' + this.ui.deleteUser] = 'onDeleteUser';
             events['click ' + this.ui.showUserList] = 'showUserList';
+            events['click '+this.ui.syncDetailes] = 'onSyncDetailes';
             return events;
 		},
 
@@ -109,6 +112,7 @@ define(function(require){
 			}
 			if(_.isUndefined(this.roleList)){
 				this.roleList = new VXRoleList();
+				this.roleList.url = "service/roles/lookup/roles";
 			}
 
 			this.bindEvents();
@@ -165,14 +169,20 @@ define(function(require){
 
         renderTable : function() {
             // this.addVisualSearch();
+            if (this.$el.find('.nav-tabs') && this.$el.find('.nav-tabs a').hasClass('active')) {
+                this.$el.find('.nav-tabs a').removeClass('active');
+            }
             if(this.showUsers){
+                this.$el.find('[data-js="users"] a').addClass('active')
                 this.renderUserTab();
             } else if(this.showGroups){
+                this.$el.find('[data-js="groups"] a').addClass('active')
                 this.renderGroupTab();
             } else {
+                this.$el.find('[data-js="roles"] a').addClass('active')
                 this.renderRoleTab();
             }
-                },
+        },
 
 		onVisibilityChange : function(e){
 			var that = this;
@@ -291,9 +301,6 @@ define(function(require){
                         this.ui.addNewUser.hide();
                         this.ui.addNewGroup.show();
                         this.$('.wrap-header').text('Group List');
-                        this.$('ul').find('[data-js="groups"]').addClass('active');
-                        this.$('ul').find('[data-js="users"]').removeClass();
-                        this.$('ul').find('[data-js="roles"]').removeClass();
             if (_.isUndefined(this.urlQueryParams) || _.isEmpty(this.urlQueryParams)) {
                 this.groupList.fetch({
 				reset:true,
@@ -320,14 +327,12 @@ define(function(require){
                         this.ui.hideShowVisibility.hide();
                         if(_.isUndefined(this.roleList) || _.isUndefined(this.urlQueryParams) || _.isEmpty(this.urlQueryParams)){
                                 this.roleList = new VXRoleList();
+                                this.roleList.url = "service/roles/lookup/roles";
                         }
                         this.ui.addNewUser.hide();
                         this.ui.addNewGroup.hide();
                         this.ui.addNewRoles.show();
                         this.$('.wrap-header').text('Role List');
-                        this.$('ul').find('[data-js="roles"]').addClass('active');
-                        this.$('ul').find('[data-js="users"]').removeClass();
-                        this.$('ul').find('[data-js="groups"]').removeClass();
                         this.renderRoleListTable();
                         this.addVisualSearch();
                         if(_.isUndefined(this.urlQueryParams) || _.isEmpty(this.urlQueryParams)) {
@@ -411,7 +416,7 @@ define(function(require){
 						fromRaw: function (rawValue, model) {
 							if(!_.isUndefined(rawValue) && rawValue.length > 0){
 								var role = rawValue[0];
-								return '<span class="label label-info">'+XAEnums.UserRoles[role].label+'</span>';
+								return '<span class="badge badge-info">'+XAEnums.UserRoles[role].label+'</span>';
 							}
 							return '--';
 						}
@@ -428,9 +433,25 @@ define(function(require){
 						fromRaw: function (rawValue, model) {
 							if(!_.isUndefined(rawValue)){
 								if(rawValue == XAEnums.UserSource.XA_PORTAL_USER.value)
-									return '<span class="label label-success">'+XAEnums.UserTypes.USER_INTERNAL.label+'</span>';
+									return '<span class="badge badge-success">'+XAEnums.UserTypes.USER_INTERNAL.label+'</span>';
 								else
-									return '<span class="label label-green">'+XAEnums.UserTypes.USER_EXTERNAL.label+'</span>';
+									return '<span class="badge badge-green">'+XAEnums.UserTypes.USER_EXTERNAL.label+'</span>';
+							}else
+								return '--';
+						}
+					}),
+					click : false,
+					drag : false,
+					editable:false,
+					sortable:false,
+				},
+				syncSource : {
+					label	: localization.tt("lbl.syncSource"),
+					cell	: Backgrid.HtmlCell.extend({className: 'cellWidth-1'}),
+					formatter: _.extend({}, Backgrid.CellFormatter.prototype, {
+						fromRaw: function (rawValue, model) {
+							if(!_.isUndefined(rawValue)){
+								return '<span class="badge badge-green">'+rawValue+'</span>';
 							}else
 								return '--';
 						}
@@ -462,16 +483,31 @@ define(function(require){
 						fromRaw: function (rawValue, model) {
 							if(!_.isUndefined(rawValue)){
 								if(rawValue)
-									return '<span class="label label-success">'+XAEnums.VisibilityStatus.STATUS_VISIBLE.label+'</span>';
+									return '<span class="badge badge-success">'+XAEnums.VisibilityStatus.STATUS_VISIBLE.label+'</span>';
 								else
-									return '<span class="label label-green">'+XAEnums.VisibilityStatus.STATUS_HIDDEN.label+'</span>';
+									return '<span class="badge badge-green">'+XAEnums.VisibilityStatus.STATUS_HIDDEN.label+'</span>';
 							}else
 								return '--';
 						}
 					}),
 					editable:false,
 					sortable:false
-				}
+				},
+				otherAttributes : {
+                    cell    : 'html',
+                    label : localization.tt("h.syncDetails"),
+                    editable:false,
+                    sortable:false,
+                    formatter: _.extend({}, Backgrid.CellFormatter.prototype, {
+                        fromRaw: function (rawValue, model) {
+                            if (!_.isUndefined(rawValue)) {
+                                return('<button data-id="syncDetailes" data-for="users" title="Sync Details" id="'+ model.get('id') +'" ><i class="fa-fw fa fa-eye"> </i></button>');
+                            } else {
+                                return '<center> -- </center>'
+                            }
+                        }
+                    }),
+                },
 			};
                         if(!SessionMgr.isSystemAdmin()){
                             delete cols.select;
@@ -537,9 +573,9 @@ define(function(require){
 						fromRaw: function (rawValue, model) {
 							if(!_.isUndefined(rawValue)){
 								if(rawValue == XAEnums.GroupSource.XA_PORTAL_GROUP.value)
-									return '<span class="label label-success">'+XAEnums.GroupTypes.GROUP_INTERNAL.label+'</span>';
+									return '<span class="badge badge-success">'+XAEnums.GroupTypes.GROUP_INTERNAL.label+'</span>';
 								else {
-									return '<span class="label label-green">'+XAEnums.GroupTypes.GROUP_EXTERNAL.label+'</span>';
+									return '<span class="badge badge-green">'+XAEnums.GroupTypes.GROUP_EXTERNAL.label+'</span>';
 								}
 							}else {
 								return '--';
@@ -551,6 +587,22 @@ define(function(require){
 					editable:false,
 					sortable:false,
 				},
+				syncSource : {
+					label	: localization.tt("lbl.syncSource"),
+					cell	: Backgrid.HtmlCell.extend({className: 'cellWidth-1'}),
+					formatter: _.extend({}, Backgrid.CellFormatter.prototype, {
+						fromRaw: function (rawValue, model) {
+							if(!_.isUndefined(rawValue)){
+								return '<span class="badge badge-green">'+rawValue+'</span>';
+							}else
+								return '--';
+						}
+					}),
+					click : false,
+					drag : false,
+					editable:false,
+					sortable:false,
+				},
 				isVisible : {
 					label	: localization.tt("lbl.visibility"),
 					cell	: Backgrid.HtmlCell.extend({className: 'cellWidth-1'}),
@@ -558,9 +610,9 @@ define(function(require){
 						fromRaw: function (rawValue, model) {
 							if(!_.isUndefined(rawValue)){
 								if(rawValue){
-									return '<span class="label label-success">'+XAEnums.VisibilityStatus.STATUS_VISIBLE.label+'</span>';
+									return '<span class="badge badge-success">'+XAEnums.VisibilityStatus.STATUS_VISIBLE.label+'</span>';
 								} else {
-									return '<span class="label label-green">'+XAEnums.VisibilityStatus.STATUS_HIDDEN.label+'</span>';
+									return '<span class="badge badge-green">'+XAEnums.VisibilityStatus.STATUS_HIDDEN.label+'</span>';
 								}
 							}else {
 								return '--';
@@ -580,11 +632,26 @@ define(function(require){
                     formatter : _.extend({}, Backgrid.CellFormatter.prototype, {
                         fromRaw : function (rawValue,model) {
                             return ('<div align="center"><button class="userViewicon" title = "View Users" data-js="showUserList" data-name="' + model.get('name')
-                                + '" data-id="' + model.id + '"<font color="black"><i class="icon-group"> </i></font></button></div>');
+                                + '" data-id="' + model.id + '"<font color="black"><i class="fa-fw fa fa-group"> </i></font></button></div>');
                         }
                     }),
-                }
-			};
+                },
+                otherAttributes : {
+                    cell    : 'html',
+                    label : localization.tt("h.syncDetails"),
+                    editable:false,
+                    sortable:false,
+                    formatter: _.extend({}, Backgrid.CellFormatter.prototype, {
+                        fromRaw: function (rawValue, model) {
+                            if (!_.isUndefined(rawValue)) {
+                                return('<button data-id="syncDetailes" data-for="groups" title="Sync Details" id="'+ model.get('id') +'" ><i class="fa-fw fa fa-eye"> </i></button>');
+                            } else {
+                                return '<center> -- </center>'
+                            }
+                        }
+                    }),
+                },
+            };
             if(!SessionMgr.isSystemAdmin()){
                 delete cols.select;
             }
@@ -713,14 +780,14 @@ define(function(require){
                         msg +='<span class="link-tag userLists span-margin setEllipsis" title="'+ _.escape(model.get('name')) +'"><a href="#!/user/'+ model.id+'">'+ _.escape(model.get('name')) + '</a></span>';
                         that.copyUserLists.push(model.get('name'));
                     });
-                    var html = '<div class="row-fluid">\
-                                    <div class="span12">\
+                    var html = '<div class="row">\
+                                    <div class="col-md-12">\
                                         <input type="text" data-id="userInput" placeholder="Search Users" class= "users-list-search">\
-                                        <div class="pull-right link-tag copyUsers btn btn-mini" title="Copy All Users Name"><i class="icon-copy"></i></div>\
+                                        <div class="pull-right link-tag copyUsers btn btn-sm" title="Copy All Users Name"><i class="fa-fw fa fa-copy"></i></div>\
                                     </div>\
                                 </div>';
                     if(totalRecords > 100){
-                        var showAllUserbtn = '<button class="btn btn-mini showMore" data-id="'+ that.groupId +'" data-id="showMore" title="Show All Users">Show All Users</button>'
+                        var showAllUserbtn = '<button class="btn btn-sm showMore" data-id="'+ that.groupId +'" data-id="showMore" title="Show All Users">Show All Users</button>'
                         infoMsg = '<div class="alert alert-warning infoWidth">'+localization.tt('msg.showInitialHundredUser')+showAllUserbtn+'</div>'
                     }
                     if(_.isEmpty(msg)){
@@ -734,6 +801,7 @@ define(function(require){
                         title   : title,
                         okText  : localization.tt("lbl.ok"),
                         allowCancel : true,
+                        focusOk : false
                     }).open();
                     modal.$el.find('.cancel').hide();
                     modal.$el.find('.copyUsers').on("click", function(e){
@@ -927,6 +995,7 @@ define(function(require){
                 if(count == numberOfUser){
                         this.collection.getFirstPage({fetch:true});
                         this.collection.selected = {};
+                        this.$el.find('table input[type="checkbox"]').prop('checked',false);
                         XAUtil.blockUI('unblock');
                         if(notDeletedUserName === "" && _.isEmpty(errorMsgForNotDeletedUsers)){
                                 XAUtil.notifySuccess('Success','User deleted successfully!');
@@ -943,13 +1012,14 @@ define(function(require){
                 if(count == numberOfGroup){
                         this.groupList.getFirstPage({fetch:true});
                         this.groupList.selected  = {};
+                        this.$el.find('table input[type="checkbox"]').prop('checked',false)
                         XAUtil.blockUI('unblock');
                         if(notDeletedGroupName === "" && _.isEmpty(errorMsgForNotDeletedGroups)){
                                 XAUtil.notifySuccess('Success','Group deleted successfully!');
                         } else {
                             var msg = "";
                             if(!_.isEmpty(notDeletedGroupName)){
-                                msg = 'Error occurred during deleting Users: '+ notDeletedGroupName.slice(0 , -2);
+                                msg = 'Error occurred during deleting Groups: '+ notDeletedGroupName.slice(0 , -2);
                             }
                             XAUtil.notifyError('Error', errorMsgForNotDeletedGroups + msg);
                         }
@@ -959,13 +1029,14 @@ define(function(require){
                 if(count == numberOfRole){
                         this.roleList.getFirstPage({fetch:true});
                         this.roleList.selected  = {};
+                        this.$el.find('table input[type="checkbox"]').prop('checked',false)
                         XAUtil.blockUI('unblock');
                         if(notDeletedRoleName === "" && _.isEmpty(errorMsgForNotDeletedRoles)){
                                 XAUtil.notifySuccess('Success','Role deleted successfully!');
                         } else {
                             var msg = "";
                             if(!_.isEmpty(notDeletedRoleName)){
-                                msg = 'Error occurred during deleting Users: '+ notDeletedRoleName.slice(0 , -2);
+                                msg = 'Error occurred during deleting Roles: '+ notDeletedRoleName.slice(0 , -2);
                             }
                             XAUtil.notifyError('Error', errorMsgForNotDeletedRoles + msg);
                         }
@@ -978,7 +1049,7 @@ define(function(require){
 			if(this.showUsers){
 				placeholder = localization.tt('h.searchForYourUser');	
 				coll = this.collection;
-				searchOpt = ['User Name','Email Address','Visibility', 'Role','User Source','User Status'];//,'Start Date','End Date','Today'];
+				searchOpt = ['User Name','Email Address','Visibility', 'Role','User Source','User Status', 'Sync Source'];//,'Start Date','End Date','Today'];
 				var userRoleList = _.map(XAEnums.UserRoles,function(obj,key){return {label:obj.label,value:key};});
                 serverAttrName  = [{text : "User Name", label :"name", urlLabel : "userName"},
                                     {text : "Email Address", label :"emailAddress", urlLabel : "emailAddress"},
@@ -986,15 +1057,16 @@ define(function(require){
                                     {text : "Visibility", label :"isVisible", 'multiple' : true, 'optionsArr' : XAUtil.enumToSelectLabelValuePairs(XAEnums.VisibilityStatus), urlLabel : "visibility"},
                                     {text : "User Source", label :"userSource", 'multiple' : true, 'optionsArr' : XAUtil.enumToSelectLabelValuePairs(XAEnums.UserTypes), urlLabel : "userSource"},
                                     {text : "User Status", label :"status", 'multiple' : true, 'optionsArr' : XAUtil.enumToSelectLabelValuePairs(XAEnums.ActiveStatus), urlLabel : "userStatus"},
+                                    {text : "Sync Source", label :"syncSource", urlLabel : "syncSource"}
                                 ];
             } else if(this.showGroups){
 				placeholder = localization.tt('h.searchForYourGroup');
 				coll = this.groupList;
-				searchOpt = ['Group Name','Group Source', 'Visibility'];//,'Start Date','End Date','Today'];
+				searchOpt = ['Group Name','Group Source', 'Visibility', 'Sync Source'];//,'Start Date','End Date','Today'];
                 serverAttrName  = [{text : "Group Name", label :"name", urlLabel : "groupName"},
                                     {text : "Visibility", label :"isVisible", 'multiple' : true, 'optionsArr' : XAUtil.enumToSelectLabelValuePairs(XAEnums.VisibilityStatus), urlLabel : "visibility"},
-                                    {text : "Group Source", label :"groupSource", 'multiple' : true, 'optionsArr' : XAUtil.enumToSelectLabelValuePairs(XAEnums.GroupTypes), urlLabel : "groupSource"}
-                                ];
+                                    {text : "Group Source", label :"groupSource", 'multiple' : true, 'optionsArr' : XAUtil.enumToSelectLabelValuePairs(XAEnums.GroupTypes), urlLabel : "groupSource"},
+                                    {text : "Sync Source", label :"syncSource", urlLabel : "syncSource"}];
             } else{
                 placeholder = localization.tt('h.searchForYourRole');
                 coll = this.roleList;
@@ -1041,6 +1113,9 @@ define(function(require){
 //										callback(XAUtil.hackForVSLabelValuePairs(XAEnums.ActiveStatus));
 										callback(that.getActiveStatusNVList());
 										break;
+									case 'Sync Source':
+										callback( _.map(XAEnums.UserSyncSource, function(obj){ return obj.label; }) );
+										break;
 									/*case 'Start Date' :
 										setTimeout(function () { XAUtil.displayDatepicker(that.ui.visualSearch, callback); }, 0);
 										break;
@@ -1086,6 +1161,34 @@ define(function(require){
 				this.ui.addNewBtnDiv.children().hide()
 			}
 		},
+        onSyncDetailes : function(e){
+            var that = this;
+            if (e.currentTarget.dataset.for =="users") {
+                var syncData = this.collection.models.find(function(m){return m.id == e.currentTarget.id});
+            } else {
+                var syncData = this.groupList.models.find(function(m){return m.id == e.currentTarget.id});
+            }
+            var SyncSourceView = Backbone.Marionette.ItemView.extend({
+                template : require('hbs!tmpl/reports/UserSyncInfo_tmpl'),
+                templateHelpers:function(){
+                   return {'syncSourceInfo' : XAViewUtils.syncUsersGroupsDetails(this)};
+                },
+                initialize: function(){
+                },
+                onRender: function(){}
+            });
+            var modal = new Backbone.BootstrapModal({
+                animate : true,
+                content     : new SyncSourceView({model : syncData}),
+                title: localization.tt("h.syncDetails"),
+                okText :localization.tt("lbl.ok"),
+                allowCancel : true,
+                escape : true,
+                focusOk : false
+            }).open();
+            modal.$el.find('.cancel').hide();
+        },
+
 		/** all post render plugin initialization */
 		initializePlugins: function(){
 		},
